/**
 * @license elementals.js (c) Jason M. Knight
 * Version 3.6.0 PRODUCTION, 23 February 2018
 * http://www.elementalsjs.com
*/

/*	
	*** MAINTAINER NOTES ***

	do NOT use "this" inside user callable objects like _.Class or _.Node as
	"this" is invalid during certain types of callbacks, such as Array.each
*/

var _;

(function(w,d,ua) {

	"use strict";

	if (!d.head) d.head = d.getElementsByTagName('head')[0];

	var
	
	// *** START local variables ***
	db = d.body,
	de = d.documentElement,
	ua_msie = ua.match(/MSIE ([0-9\.]+)/i),
	ua_edge = ua.match(/Edge\/([0-9\.]+)/i),
	ua_rv = ua.match(/rv:([0-9\.]+)/i),
	blockDisplays = ['block', 'list-item', 'table', 'table-caption', 'table-row'],
	escMap = {
		'"' : '\\"',
		'\\' : '\\\\',
		'\b' : '\\b',
		'\f' : '\\f',
		'\n' : '\\n',
		'\r' : '\\r',
		'\t' : '\\t'
	},
	escRegex = /[\\\u0000-\u001F\u0022\u2028\u2029]/g,
	htmlSpecialCharsMap = {
		'&' : '&amp;',
		'<' : '&lt;',
		'>' : '&gt;',
		'"' : '&quot;',
		"'" : '&#039;'
	},
	htmlSpecialCharsRegex = /[&<>"']/g,
	condenseRegex = /\[\s\xA0]+/g,
	stripCamelRegex = /([A-Z])/g,
	toCamelRegex = /(\W[\w])/g,
	trimRegex = /^[\s\xA0]+|[\s\xA0]+$/g,
	trimLeftRegex = /^[\s\xA0]+/,
	trimRightRegex = /[\s\xA0]+$/,
	EventAdd,
	globalTemp,
	t_vmb = '"value" must be ',
	
	// *** END local variables ***

	// *** START local objects ***

	iWrite = {
		after : function(e, content) {
			return e.parentNode.appendChild(domToObject(content), e.nextSibling);
		},
		before : function(e, content) {
			return e.parentNode.insertBefore(domToObject(content), e);
		},
		first : function(e, content) {
			return e.insertBefore(domToObject(content), e.firstChild);
		},
		last : function(e, content) {
			return e.appendChild(domToObject(content));
		},
		rewrite : function(e, content) {
			_.Node.flush(e);
			return _.Node.write(e, content);
		}
	}; // iWrite
	
	_ = {

		BROWSER_PREFIXES : ['ms', 'moz', 'o', 'webkit'],
		
		Class : {

			add : function(element, cName) {
				return cName instanceof Array ?
					cName.callbackOr(element, _.Class.add) :
					_.Class.exists(element, cName) ? true :
					(element.className = (element.className + ' ' + cName).condense());
			}, // _.Class.add

			exists : function(element, cName) {
				return cName instanceof Array ?
					cName.callbackAnd(element, _.Class.exists) :
					newCNR(cName).test(element.className);
			}, // _.Class.exists

			remove : function(element, cName) {
				if (cName instanceof Array) return cName.callbackOr(element, _.Class.remove);
				if (!_.Class.exists(element, cName)) return false;
				element.className = element.className.replace(newCNR(cName), '').condense();
				return true;
			}, // _.Class.remove

			swap : function(element, oldClass, newClass) {
				oldClass = _.Class.remove(element, oldClass);
				newClass = _.Class.add(element, newClass);
				return oldClass || newClass;
			}, // _.Class.swap

			toggle : function(element, cName) {
				if (_.Class.remove(element, cName)) return false;
				_.Class.add(element, cName);
				return true;
			} // _.Class.toggle

		}, // _.Class

		Compare : {

			string : function(a, b) {
				return _.safeString(a).localeCompare(_.safeString(b));
			},

			stringI : function(a, b) {
				return _.safeString(a, true).localeCompare(_.safeString(b, true));
			},

			numeric : function(a, b) {
				return (Number(a) || 0) - (Number(b) || 0);
			}

		}, // _.Compare

		dataGet : function(element, name, splitResult, temp) {
			return (temp = element.getAttribute(name = 'data-' + name)) === null ? false :
				temp === '' ? element.hasAttribute && element.hasAttribute(name) :
					splitResult ? temp.split(',') : temp;
		}, // _.dataGet

		EMPTY_TAGS : [
			'AREA', 'BASE', 'BR', 'COL', 'COMMAND', 'EMBED', 'HR', 'IMG',
			'INPUT', 'KEYGEN', 'LINK', 'META', 'PARAM', 'SOURCE', 'TRACK',
			'WBR'
		], // _.EMPTY_TAGS
		
		IS_IE : ua_msie ? ua_msie[1] : (/Trident/i.test(ua) ? ua_rv[1] : false),
		IS_EDGE : (ua_edge && !_.IS_IE) ? ua_edge[1] : false,

		ajax : function(params) {
			switch (typeof params) {
				case 'function':
					params = { 'status' : { '200' : params } };
				case 'object':
					return (new eXMLHttp(params)).result;
			}
			tnt('Invalid Ajax declaration, parameter "handler" not a function or object!');
		}, // _.ajax

		Cookies : {

			'delete' : function(name, path, domain) {
				_.Cookies.get(name) && _.Cookies.set(name, '', 0, path, domain);
			}, // _.Cookies.delete
			
			get : function(name) {
				return name && (name = d.cookie.match(
					new RegExp('(^|;)\\s*' + name + '\\s*=\\s*([^;]*)')
				) ? unescape(name[2]) : null);
			}, // _.Cookies.get

			set : function(name, value, time, path, domain, secure) {
				if (
					'undefined' == typeof time ||
					time === false
				) time = 34158207600000; // 6 June 3052, 10:00AM EST
				if (Number.isInteger(time)) time = new Date(time);
				if (time instanceof Date) time = time.toGMTString();
				_.Throw.nonType(time, 'string');
				d.cookie = name + '=' + escape(value) +
					'; expires=' + time +
					(path ? '; path=' + path : '') +
					(domain ? '; domain=' + domain : '') +
					(secure ? '; secure' : '');
			}, // _.Cookies.set

			setAfter : function(name, value, time, path, domain, secure) {
				_.Cookies.set(name, value, time ? new Date().getTime() + time : false, path, domain, secure);
			} // _.Cookies.setAfter

		}, // _.Cookies

		Event : {

			add : function(e, value, callback, prevent) {
				new EventAdd(e, value, callback, true == prevent ? 1 : (prevent || 0));
			},

			PREVENT_BUBBLE : 1,
			PREVENT_PROPAGATION : 3

		}, // _.Event

		formSerialize : function(formElement) {

			_.Throw.notTagName(formElement, 'FORM');
			
			var result = '', temp;

			function add(name, value) {
				if (value && ('' !== value)) result +=
					(result ? '&' : '') +
					encodeURIComponent(name) +
					'=' +
					encodeURIComponent(value);
			}

			for (var i = 0, e; e = formElement.elements[i]; i++) {
				if (e.name) switch (e.tagName.toUpperCase()) {
					case 'INPUT':
						switch (e.type) {
							case 'file' :
								// to-do, add serialization for browsers that support it
								tnt('Cannot Serialize a <input type="file">');
							case 'checkbox':
							case 'radio':
								if (!e.checked) continue;
						}
					case 'BUTTON':
					case 'TEXTAREA':
						add(e.name, e.value);
					break;
					case 'SELECT':
						temp = e.options[e.selectedIndex];
						add(e.name, temp.value || _.Node.text(temp));
				}
			}

			return result;

		}, // _.formSerialize

		Load : {

			style : function(href, media) {
				return media ? ( media = media.toLowerCase(), "all" == media && _.Log.warn(
					'media="all" is NOT recommended - ' + href
				)) : _.Log.warn(
					'no media type declared for ' + href + ', defaulting to "screen,projection,tv"'
				), _.make('link', {
					rel : 'stylesheet',
					href : href,
					media : media || 'screen,projection,tv',
					last : d.head
				});
			}, // _.Load.style

			script : function(src, attr) {
				return _.make('script@' + src, attr);
			} // _.Load.script

		}, // _.Load

		Log : {

			alert : function(message) {
				alert(message);
			}, // _.Log.alert

			append : function(message) {
				_.make('br', { last : db });
				_.Node.write(db, message);
			}, // _.Log.append

			error : function(message) {
				throw new Error(message);
			}, // _.Log.error

			log : function(message) {
				w.console ? console.log(message) : _.Log.append(message);
			}, // _.Log.log

			warn : function(message) {
				w.console ? console.warn(message) : _.Log.append(message);
			} // _.Log.warn

		}, // _.Log
		
		make : function(selector, attributes, e) {
			if (
				!selector ||
				typeof selector == 'object' ||
				typeof selector == 'function' ||
				selector instanceof Array
			) tnt('selector must be a non-empty string');
			selector = String(selector);
			if (selector[0] == '/') e = d.createTextNode(selector.substr(1));
			else {
				selector = selector.splitSelector();
				e = d.createElement(selector.tag);
				selector.classes && (e.className = selector.classes);
				selector.id && (e.id = selector.id);
				if (selector.special) switch (selector.tag.toLowerCase()) {
					case 'a': e.href = selector.special; break;
					case 'button':
					case 'input': e.type = selector.special; break;
					case 'img': e.src = selector.special; break;
					case 'option': e.value = selector.special; break;
					case 'script':
						e.src = selector.special;
						if (!attributes) attributes = {};
						if (!Object.keysDoIntersect(iWrite, attributes)) attributes.last = db;
						break;
					case 'th': e.scope = selector.special; break;
					case 'td': e.headers = selector.special; break;
				}
				if (selector.content) switch (selector.tag.toLowerCase()) {
					case 'img': e.alt = selector.content; break;
					case 'option':
					case 'input':
					case 'textarea': e.value = selector.content; break;
					default: _.Node.write(e, selector.content);
				}
			}
			if (attributes instanceof Array) return _.Node.write(e, attributes);
			else if ('object' == typeof attributes) {
				var write = false;
				for (var name in attributes) {
					var value = attributes[name];
					if ('on' == name.substr(0, 2)) _.Event.add(e, name.substr(2), value);
					else if (iWrite[name]) {
						if (write) throw new Error(
							'Multiple write methods passed to _.make, only one of "after, before, first, last, replace" may be present in attribute object.'
						);
						write = name;
					} else switch (name) {
						case 'content':
							_.Node.write(e, value);
							break;
						case 'style':
							for (var i in value) e.style[i] = value[i];
							break;
						case 'type':
							// legacy IE can't handle setting type on Element directly
							e.setAttribute(name, value);
							break;
						default:
							// but legacy IE screws up setAttribute a LOT on other attributes so...
							e[name] = value;
					}
				}
				if (write) _.Node.write(attributes[write], e, write);
			} else if (attributes && e.nodeType == 1) iWrite.last(attributes);
			return e;
		}, // _.make

		Metrics : {

			absoluteX : function(element) {
				return element.getBoundingClientRect().left + _.Metrics.Scroll.x();
			}, // _.Metrics.absoluteX

			absoluteY : function(element) {
				return element.getBoundingClientRect().top + _.Metrics.Scroll.y();
			}, // _.Metrics.absoluteY

			height : function(element) {
				return !element || element == w ? (
					w.innerHeight || de.clientHeight || db.clientHeight
				) : element.clientHeight;
			}, // _.Metrics.height

			Scroll : {

				height : function(element) {
					return (!element || element == w ? db : element).scrollHeight;
				}, // _.Metrics.Scroll.height

				maxX : function(element) {
					return _.Metrics.Scroll.width(element) - _.Metrics.width(element);
				}, // _.Metrics.Scroll.maxX

				maxY : function(element) {
					return _.Metrics.Scroll.height(element) - _.Metrics.height(element);
				}, // _.Metrics.Scroll.maxY

				width : function(element) {
					return (!element || element == w ? db : element).scrollWidth;
				}, // _.Metrics.Scroll.width

				x : function(element) {
					return !element || element == w ? (
						w.scrollX || w.pageXOffset || db.scrollLeft
					) : element.scrollLeft;
				}, // _.Metrics.Scroll.x

				y : function(element) {
					return !element || element == w ? (
						w.scrollY || w.pageYOffset || db.scrollTop
					) : element.scrollTop;
				} // _.Metrics.Scroll.y

			}, // _.Metrics.Scroll

			width : function(element) {
				return !element || element == w ? (
					w.innerWidth || de.clientWidth || db.clientWidth
				) : element.clientWidth;
			}, // _.Metrics.width
			
			windowHeight : function() {
				return de.clientHeight;
			}, // _.Metrics.windowHeight

			windowWidth : function() {
				return de.clientWidth;
			} // _.Metrics.windowWidth

		}, // _.Metrics

		Node : {

			addCopyButton : function(
				element, buttonContent, buttonParentElement, placement
			) {
				makeSelectionButton(
					element,
					buttonContent || 'Copy',
					buttonParentElement,
					placement,
					function(a) {
						_.Select.node(a.currentTarget['data-eNodeSelectTarget']);
						_.Select.toClipboard();
					}
				);
			}, // _.Node.addCopyButton

			addSelectButton : function(
				element, buttonContent, buttonParentElement, placement
			) {
				makeSelectionButton(
					element,
					buttonContent || 'Select',
					buttonParentElement,
					placement,
					function(e) {
						_.Select.node(e.currentTarget['data-eNodeSelectTarget']);
					}
				);
			}, // _.Node.addSelectButton

			countAll : function(element, nodeMask) {
				return nCountBy(element, nodeMask, 'eachAll');
			}, // _.Node.countAll

			countChildren : function(element, nodeMask) {
				return nCountBy(element, nodeMask, 'eachChild');
			}, // _.Node.countChildren

			eachAll : function(element, callback, nodeMask) {
				if (element = element.firstChild) do {
					if (
						(_.Node.mask(element, nodeMask) && !callback(element)) ||
						(element.firstChild && !_.Node.eachAll(element, callback, nodeMask))
					) return false;
				} while (element = element.nextSibling);
				return true;
			}, // _.Node.eachAll
			
			eachChild : function(element, callback, nodeMask) {
				if (element = element.firstChild) do {
					if (_.Node.mask(element, nodeMask) && !callback(element)) return false;
				} while (element = element.nextSibling);
				return true;
			}, // _.Node.eachChild
			
			first : function(element, nodeMask) {
				return _.Node.walk(element, nodeMask, 'firstChild', 'nextSibling');
			}, // _.Node.first

			flush : function(element) {
				while (element.firstChild) element.removeChild(element.firstChild);
			}, // _.Node.flush

			last : function(element, nodeMask) {
				return _.Node.walk(element, nodeMask, 'lastChild', 'previousSibling');
			}, // _.Node.last

			mask : function(element, nodeMask) {
				return nodeMask === 0 ? true :
					isNaN(nodeMask = nodeMask || 1) ? (
						element.nodeType == 1 &&
						element.tagName.toUpperCase() == nodeMask.toUpperCase()
					) : ((
						element.nodeType == 0 ? 0 : 1 << (element.nodeType - 1)
					) & nodeMask) > 0;
			}, // _.Node.mask

			next : function(element, nodeMask) {
				return _.Node.walk(element, nodeMask, 'nextSibling', 'nextSibling');
			}, // _.Node.next

			nextRemove : function(element, nodeMask, temp) {
				temp = _.Node.next(element, nodeMask);
				_.Node.remove(element);
				return temp;
			}, // _.Node.nextRemove

			prev : function(element, nodeMask) {
				return _.Node.walk(element, nodeMask, 'previousSibling', 'previousSibling');
			}, // _.Node.prev

			prevRemove : function(element, nodeMask, temp) {
				temp = _.Node.prev(element, nodeMask);
				_.Node.remove(element);
				return temp;
			}, // _.Node.prevRemove

			remove : function(element) {
				element.parentNode.removeChild(element);
			}, // _.Node.remove

			replace : function(oldNode, newNode) {
				return oldNode.parentNode.replaceChild(domToObject(newNode));
			}, // _.Node.replace
			
			/*
				Why not innerText or textContent for _.Node.text? HERE'S WHY:
				http://perfectionkills.com/the-poor-misunderstood-innerText/
				Hardcode this s*** so it's CONSISTENT! Not rocket science.
			*/
			
			text : function(element) {
				var result = '';
				_.Node.eachAll(element, function(node) {
					result += node.nodeValue + ' '; 
					return true; 
				}, _.Node.mask.TEXT);
				return result.condense();
			},
			
			textFormatted : function(element) {
				var result = '';
				var blockAfter = false;
				_.Node.eachAll(element, function(e) {
					switch (e.nodeType) {
						case 1:
							if (
								e.tagName == 'input' ||
								e.tagName == 'textArea'
							) {
								result += e.nodeValue.condense() + ' ';
								break;
							}
							var style = getComputedStyle(e);
							if (
								style.visibility == 'hidden' ||
								style.display == 'none'
							) break;
							if (
								blockDisplays.includes(style.display) ||
								e.tagName == 'BR'
							) {
								if (blockAfter) result = result.trimRight() + "\r\n";
								blockAfter = true;
							}
							break;
						case 3:
							result += e.nodeValue.condense() + ' ';
					}
					return true;
				}, 0);
				return result.trim();
			}, // _.Node.textFormatted
			
			walk : function(element, nodeMask, firstMethod, nextMethod, temp) {
				if (temp = element[firstMethod]) do {
					if (_.Node.mask(temp, nodeMask)) return temp;
				} while (temp = temp[nextMethod]);
				return false;
			}, // _.Node.walk
			
			write : function(element, content, placement) {
				if (!placement || !iWrite[placement]) placement = 'last';
				if (content instanceof Array) {
					for (var i = 0, child; child = content[i]; i++) {
						if (child instanceof Array) {
							if (child[1] instanceof Array) child[1] = { content : child[1] };
							else if (
								'undefined' != typeof child[1] &&
								"object" != typeof child[1]
							) child[1] = { content : child[1] };
							if (!child[1]) child[1] = { last : element };
							else if (!Object.keysDoIntersect(iWrite, child[1])) child[1].last = element;
							child = _.make(child[0], child[1]);
							switch (placement) {
								case 'first':
								case 'before':
								case 'replace':
								case 'rewrite':
									placement = 'after';
								case 'after':
									element = child;
							}
						} else iWrite[placement](element, child);
					}
				} else iWrite[placement](element, content);
			} // _.Node.write
			
		}, // _.Node
		
		safeString : function(stringish, toLowerCase) {
			stringish = (
				'undefined' == typeof stringish ||
				(!stringish && stringish !== 0)
			) ? '' : String(stringish).trim();
			return toLowerCase ? stringish.tolowercase() : stringish;
		}, // _.safeString

		Select : {

			all : function() {
				_.Select.node(db);
			} // _.Select.all

		}, // _.Select

		Throw : {

			isLooseFalse : function(value) {
				if (!value) tnt('Variable is loose false, truthy expected.');
			}, // _.Throw.isLooseFalse

			isNaN : function(value) {
				_.Throw.isNull(value);
				if (value != value) tnr(t_vmb + 'a number');
			}, // _.Throw.isNaN

			isNull : function(value) {
				if (null == value || 'undefined' == typeof value) tnt(
					'Variable Null or Undefined'
				);
			}, // _.Throw.isNull
			
			nonElement : function(element) {
				_.Throw.isNull(element);
				if (element.nodeType !== 1) tnt(
					element + ' is not a DOM Element'
				);
			}, // _.Throw.nonElement

			nonInstance : function(object1, object2) {
				_.Throw.isNull(object1);
				if (!(object1 instanceof object2)) tnt(
					object1 + ' is not an instance of ' + object2
				);
			}, // _.Throw.nonInstance

			nonType : function(value, type) {
				_.Throw.isNull(value);
				if (type !== typeof value) tnt(
					value + ' is not a "' + type + '" type'
				);
			}, // _.Throw.nonType

			notTagName : function(element, tagName) {
				_.Throw.isNull(element);
				if (
					1 != element.nodeType ||
					element.tagName.toUpperCase() != tagName.toUpperCase()
				) tnt(
					element + ' is not a <' + tagName + '>'
				);
			}, // _.Throw.notTagName

			range : function(value, min, max) {
				_.Throw.isNaN(value);
				if (value == Infinity) tnr(
					t_vmb + 'less than Inifinty'
				);
				if (!isNaN(min) && value < min) tnr(
					t_vmb + 'greater than or equal to ' + min
				);
				if (!isNaN(max) && value > max) tnr(
					t_vmb + 'less than or equal to ' + max
				);
			}, // _.Throw.range
			
			'type' : function(value, type) {
				if (type == typeof value) tnt(
					value + ' cannot be type "' + type + '"'
				);
			} // _.Throw.type

		}, // _.Throw
		
		USort : {

			element : function(e, callback) {
				globalTemp = [];
				switch (e.tagName.toUpperCase()) {
					case 'TBODY' :
						_.Node.eachChild(e, USortTR, 'TR');
						break;
					case 'UL':
					case 'OL':
						_.Node.eachChild(e, USortLI, 'LI');
						break;
					default: tnt(e + ' is not a TBODY, UL, or OL');
				}
				e['data-USortElementSorted'] = true;
				globalTemp.sort(callback);
				_.Node.flush(e);
				for (var i = 0, j; j = globalTemp[i]; i++) e.appendChild(j.tag);
				globalTemp = null;
			}, // _.USort.element

			restore : function(e) {
				switch (e.tagName.toUpperCase()) {
					case 'TBODY':
					case 'UL':
					case 'OL':
						if (e['data-USortElementSorted']) _.USort.element(e, originalSort);
						return;
				}
				tnt(e + ' is not a TBODY, UL, or OL');
			} // _.USort.restore

		} // _.USort

	}; // _
	
	// *** END local objects

	/*
		*** START local functions ***

		These functions are invisible outside of this self-instancing
		function and are meant for internal use by elementals ONLY.
	*/
	
	function browserFix(target, methodName, altMethodName) {
		if (target[methodName]) return true;
		altMethodName || (altMethodName = methodName.charAt(0).toUpperCase() + methodName.substr(1));
		for (var i = 0, j, k; j = _.BROWSER_PREFIXES[i]; i++)
			if (target[k = j + altMethodName]) return target[methodName] = target[k], true;
		return false;
	} // browserFix
	
	function domToObject(value) {
		return 'object' == typeof value ? value : d.createTextNode(value);
	} // domToObject
	
	function eXMLHttp(params, ajax) {

		if (!(ajax = this.result = w.XMLHttpRequest ? new XMLHttpRequest() : (
			newAX('Msxml2.XMLHTTP.6.0') ||
			newAX('Msxml2.XMLHTTP.3.0') ||
			newAX('Microsoft.XMLHTTP')
		))) throw new Error('Unable to create XMLHttpRequest or ActiveX XMLHTTP');

		if (params.progress) ajax.onprogress = params.progress;

		ajax.getJSON = function() {
			return JSON.parse(this.responseText);
		};

		ajax.getXMLValueById = function(id) {
			return (id = this.responseXML.getElementById(id)) ? _.Node.text(id) : null;
		};

		ajax.getXMLValuesByTagName = function(tagName, limit, elements) {
			if (!(elements = this.responseXML.getElementsByTagName(tagName))) return false;
			if (true === limit) return _.Node.text(elements[0]);
			if (!limit || elements.length < limit) limit = elements.length;
			for (var result = [], i = 0; i < limit; i++) result.push(_.Node.text(elements[i]));
			return result;
		};

		ajax.onreadystatechange  = function() {
			if (
				params.state &&
				params.state[ajax.readyState]
			) return params.state[ajax.readyState](ajax, params);
			if (4 == ajax.readyState) {
				if (params.status) {
					if (params.status[ajax.status]) return params.status[ajax.status](ajax, params);
					if (params.status.catchAll) return params.status.catchAll(ajax, params);
				}
				_.Log[params.onStatusLog || 'warn']('Unhandled AJAX Status ' + ajax.status);
			} else if (params.onReadyLog) _.Log[params.onReadyLog]('Unhandled AJAX readyState ' + ajax.readyState);
		};

		if (params.request) {
			ajax.open(params.request[0], params.request[1]);
			if (params.mimeType) ajax.overrideMimeType(params.mimeType);
			if (params.headers)
				for (var i in params.headers)
					ajax.setRequestHeader(i, params.headers[i]);
			ajax.send(params.request[2]);
		}

	} // eXMLHttp

	// var so we can overload it
	var makeSelectionButton = function(targetElement, buttonContent, buttonParent, placement, callback, temp) {
		_.Throw.nonElement(targetElement);
		if (_.NON_CONTENT_TAGS.indexOf(targetElement.tagName.toUpperCase()) !== -1)
			_.Log.Error('Cannot create selection button for <' + targetElement.tagName + '>');
		temp = {
			type : 'button',
			content : buttonContent,
			onclick : callback,
			'data-eNodeSelectTarget' : targetElement
		};
		temp[placement || 'before'] = buttonParent || targetElement;
		_.make('button.selectContents', temp);
	}; // makeSelectionButton

	function newAX(axName) {
		try { return new activeXObject(axName); } catch (axName) { return false; }
	} // newAX
	
	function newCNR(cName) {
		return new RegExp('(\\s|^)' + cName + '(\\s|$)');
	}

	function nCountBy(element, nodeMask, method, i) {
		i = 0;
		_.Node[method](element, function() { return i++; }, nodeMask);
		return i;
	} // nCountBy

	function originalSort(a, b) {
		return _.Compare.numeric(
			a.tag.getAttribute('data-USortOriginalPlace'),
			b.tag.getAttribute('data-USortOriginalPlace')
		);
	} // originalSort

	function polyfillAddPrototype(targetObject, sources) {
		for (var i in sources)
			targetObject.prototype[i] || (targetObject.prototype[i] = sources[i]);
	} // polyfillAdd
	
	function polyfillAddStatic(targetObject, sources) {
		for (var i in sources)
			targetObject[i] || (targetObject[i] = sources[i]);
	} // polyfillAddStatic
	
	function selectorStringChunker(str) {
		this.remains = str;
		this.next = function(match, i, r) {
			if ((i = this.remains.indexOf(match)) == -1) return false;
			r = this.remains.substr(i + match.length);
			this.remains = this.remains.substr(0, i);
			return r.length == 0 ? false : r;
		}
	}
	
	function test_funcBVoid(a, b) {
		_.Throw.nonType(a, 'function');
		return 'undefined' == typeof b ? void 0 : b;
	}
	
	function test_nullLength(arr, b, value) {
		_.Throw.isNull(arr);
		return arr.length == 0 ? false : (
			'undefined' == typeof b || (b < 0 && (b += this.length) < 0)
		) ? value : b;
	}
	
	function tnt(message) { throw new typeError(message); }
	function tnr(message) { throw new rangeError(message); }
	
	function notSetSet(e, attrName, value) {
		e.getAttribute(attrName) || e.setAttribute(attrName, value);
	}
	
	function USortTR(tr, result) {
		notSetSet(tr, 'data-USortOriginalPlace', globalTemp.length);
		result = [];
		_.Node.eachChild(tr, function(a) {
			return result.push(_.Node.text(a)), true;
		});
		globalTemp.push({ tag : tr.cloneNode(true), data : result });
		return true;
	} // USortTR

	function USortLI(li) {
		notSetSet(li, 'data-USortOriginalPlace', globalTemp.length);
		globalTemp.push({ tag : li.cloneNode(true), data : _.Node.text(li) });
		return true;
	} // USortLI
	
	// *** END local functions ***
	
	iWrite.replace = _.Node.replace;
	
	// *** START static and generated underscore properties ***

	_.Node.mask.ALL        = 0;
	_.Node.mask.ELEMENT    = 1;
	_.Node.mask.TEXT       = 4;
	_.Node.mask.PROCESSING = 64;
	_.Node.mask.COMMENT    = 128;
	_.Node.mask.DOCUMENT   = 256;
	_.Node.mask.DOCTYPE    = 512;
	_.Node.mask.FRAGMENT   = 1024;
	
	_.NON_CONTENT_TAGS = _.EMPTY_TAGS.concat([
		'AUDIO', 'COLGROUP', 'HEAD', 'MAP', 'METER', 'PROGRESS',
		'OBJECT', 'SCRIPT', 'STYLE', 'TEXTAREA', 'VIDEO'
	]); // _.NON_CONTENT_TAGS

	// *** END static and generated underscore properties ***

	// START browser specific _.Event.prevent and local EventAdd object
	if (w.addEventListener) {
		_.Event.prevent = function(e) {
			if (e.stopPropagation) e.stopPropagation();
			if (e.preventDefault) e.preventDefault();
		};
		EventAdd = function(element, event, callback, captureLevel) {
			element.addEventListener(event, function(e) {
				callback(e);
				if (captureLevel > 1) _.Event.prevent(e);
			}, captureLevel == 1);
		};
	} else {
		_.Event.prevent = function(e) {
			e.cancelBubble = true;
			e.returnValue = false;
		};
		EventAdd = function(element, event, callback, captureLevel) {
			element.attachEvent('on' + event, function(e) {
				e = w.event;
				e.target = e.srcElement;
				e.currentTarget = element;
				callback(e);
				if (captureLevel > 0) {
					e.cancelBubble = true;
					if (captureLevel > 1) e.returnValue = false;
				}
			});
		};
	}
	// END browser specific _.Event.prevent and local EventAdd object
	
	// START browser specific _.Select.node and _.Select.none
	if (d.selection) {
		_.Select.node = function(element, temp) {
			temp = db.createTextRange();
			temp.moveToElementText(element);
			temp.select();
			return true;
		};
		_.Select.none = function() {
			d.selection.empty();
			return true;
		};
	} else if (w.getSelection) {
		_.Select.node = function(element, tempRange, tempSel) {
			tempSel = w.getSelection();
			tempRange = d.createRange();
			tempRange.selectNodeContents(element);
			tempSel.removeAllRanges();
			tempSel.addRange(tempRange);
			return true;
		};
		_.Select.none = function() {
			w.getSelection().removeAllRanges();
			return true;
		};
	} else {
		_.Select.node =
		makeSelectionButton = function() {
			_.Log.warn('selection of content not supported');
			return false;
		};
		_.Select.none = function() {
			_.Log.warn('de-selection of content not supported');
			return false;
		};
	}
	// END browser specific _.Select.node and _.Select.none

	// START browser specific _.writeToClipboard, _.Select.toClipBoard
	if (
		w.clipboardData &&
		w.clipboardData.setData
	) {
		_.writeToClipboard = function(a) {
			w.clipboardData.setData('Text', String(a));
			return true;
		};
		_.Select.toClipboard = function() {
			_.writeToClipboard(
				'function' == typeof d.getSelection ?
				String(d.getSelection()) :
				d.selection.createRange().text
			);
			return true;
		};
	} else if (d.queryCommandSupported('copy')) {
		_.writeToClipboard = function(a, b) {
			b = _.make('textarea', { content : String(a), last : db });
			b.select();
			d.execCommand('copy');
			_.Node.remove(b);
			return true;
		};
		_.Select.toClipboard = function() {
			d.execCommand('copy');
			return true;
		};
	} else {
		_.writeToClipboard =
		_.Select.toClipboard = function() {
			_.Log.warn('This browser does not allow JavaScript to access the clipboard');
			return false;
		};
	}
	// END browser specific _.writeToClipboard, _.Select.toClipBoard

	// START browser specific _.Query and _.Query.all
	if ('function' == typeof d.querySelectorAll) {
		_.Query = function(element, selector) {
			return element.querySelector(selector);
		};
		_.Query.all = function(element, selector) {
			// [].slice makes it return an ARRAY
			return [].slice.call(element.querySelectorAll(selector));
		};
	} else {
		var tempStyle = d.createStyleSheet();
		_.Query = function(element, selector, temp) {
			temp = _.Query.all(element, selector, 1);
			return (temp && temp.length > 0) ? temp[0] : null;
		};
		_.Query.all = function(element, selector, limit, results) {
			if (!limit) limit = 32767;
			results = [];
			tempStyle.addRule(selector, 'fu:bar');
			function callback(e) {
				if ((limit > 0) && (e.currentStyle.fu === 'bar')) results.push(e);
				return --limit > 0;
			}
			_.Node.eachAll(element, callback);
			tempStyle.removeRule();
			return results;
		};
	}
	// END browser specific _.Query and _.Query.all

	// *** END underscore methods that require locals ***

	// *** START polyfills and browser object extensions ***
	
	if (!Array.prototype.reduce) {
		var arrayReducer = function(a, b, arr, right, i, j) {
			_.Throw.nonType(a, 'function');
			j = this.length;
			i = 0;
			if ('undefined' == typeof b) {
				if (!this.strLen) tnt('Reduce of empty array with no initial value');
				if (this.strLen == 1) return this[0];
				if (right) b = arr[--j]; else { b = arr[0]; i = 1; }
			}
			if (right) while (j > 0) b = a(b, this[--j], j, this);
			else while (i < j) b = a(b, this[i], i++, this);
			return b;
		};
	}

	polyfillAddPrototype(Array, {

		callbackAnd : function(a, b) {
			for (var i = 0, l = this.length; i < l; i++)
				if (!b(a, this[i])) return false;
			return true;
		}, // Array.callbackAnd

		callbackAndAll : function(a, b) {
			for (var i = 0, l = this.length, r = false; i < l; i++)
				r = b(a, this[i]) && r;
			return r;
		}, // Array.callbackAndAll

		callbackOr : function(a, b) {
			for (var i = 0, l = this.length, r = false ; i < l; i++)
				r = b(a, this[i]) || r;
			return r;
		}, // Array.callbackOr

		every : function(a, b) {
			b = test_funcBVoid(a, b);
			for (var i = 0, j = this.length; i < j; i++)
				if (!a.call(b, this[i], i, this)) return false;
			return true;
		}, // Array.every

		fill : function(a, b, c) {
			_.Throw.isNull(this);
			if ('undefined' == typeof b) b = 0;
				else if (b < 0) b += this.length;
			if ('undefined' == typeof c) c = this.length;
			for (var i = b; i < c; i++) this[i] = a;
			return this;
		}, // Array.fill

		filter : function(a, b) {
			b = test_funcBVoid(a, b);
			for (var r = [], i = 0, j = this.length; i < j; i++)
				if (a.call(b, this[i], i, this)) r.push(this[i]);
			return r;
		}, // Array.filter

		find : function(a, b) {
			b = test_funcBVoid(a, b);
			for (var i = 0, j = this.length; i < j; i++)
				if (a.call(b, this[i], i, this)) return this[i];
		}, // Array.find

		findIndex : function(a, b) {
			b = test_funcBVoid(a, b);
			for (var i = 0, j = this.length; i < j; i++)
				if (a.call(b, this[i], i, this)) return i;
		}, // Array.findIndex

		forEach : function(a, b) {
			b = test_funcBVoid(a, b);
			for (var i = 0, j = this.length; i < j; i++)
				a.call(b, this[i], i, this);
		}, // Array.forEach

		includes : function(a, b) {
			if ((b = test_nullLength(this, b, 0)) === false) return false;
			var c = a != a;
			for (var j = this.length; b < j; b++)
				if (a == this[b] || (c && this[b] != this[b])) return true;
			return false;
		}, // Array.includes

		indexOf : function(a, b) {
			if ((b = test_nullLength(this, b, 0)) === false) return false;
			for (var c = this.length; b < c; b++) if (this[b] == a) return b;
			return -1;
		}, // Array.indexOf

		isArray : function(a) {
			return Object.prototype.toString.call(a) === '[object Array]';
		}, // Array.isArray

		lastIndexOf : function(a, b) {
			if ((b = test_nullLength(this, b, this.length)) === false) return false;
			while (b > 0) if (this[--b] == a) return b;
			return -1;
		}, // Array.lastIndexOf

		map : function(a, b) {
			b = test_funcBVoid(a, b);
			for (var r = [], i = 0, j = this.length; i < j; i++)
				r.push(a.call(b, this[i], i, this));
			return r;
		}, // Array.map
		
		reduce : function(a, b) {
			return arrayReducer(a, b, this);
		}, // Array.reduce

		reduceRight : function(a, b) {
			return arrayReducer(a, b, this, true);
		}, // Array.reduceRight

		some : function(a, b) {
			b = test_funcBVoid(a, b);
			for (var i = 0, j = this.length; i < j; i++)
				if (a.call(b, this[i], i, this)) return true;
			return false;
		} // Array.some

	}); // polyfill Array
	
	// 'of' has to go on it directly.
	polyfillAddStatic(Array, {
		'of' : function() {
			_.Throw.isNull(this);
			return Array.prototype.slice.call(arguments);
		}
	});
	
	polyfillAddStatic(Math, {
	
		// 'constants' 
		RAD : Math.PI / 180,
		QUD : Math.PI / 360,
		DEG : 180 / Math.PI,
		
		// methods
		
		acosh : function(a) {
			return Math.log(a + Math.sqrt(a * a - 1));
		}, // Math.acosh
		
		asinh : function(a) {
			return a === -Infinity ? a : Math.log(a + Math.sqrt(a * a + 1));
		}, // Math.asinh
		
		atanh : function(a) {
			return Math.log((1 + a) / (1 - a)) / 2;
		}, // Math.atanh
		
		cbrt : function(a, b) {
			b = Math.pow(Math.abs(a), 1 / 3);
			return a < 0 ? -b : b;
		}, // Math.cbrt
		
		clz32 : function(a) {
			return !a ? 32 : 31 - Math.floor(Math.log(a >>> 0) * Math.LOG2E);
		},
		
		clz64 : function(a) {
			return !a ? 64 : 63 - Math.floor(Math.log(a >>> 0) * Math.LOG2E);
		},
		
		cosh : function(a) {
			return ((a = Math.exp(a)) + 1 / a) / 2;
		},
		
		/*
			I am NOT polyfilling Math.expm1, type the -1 you lazy ****'s!
			Same for Math.log10, Math.logP, Math.log2
			Why the **** do those even EXIST?!?
			
			Skipping Math.fround, MAYBE later.
		*/
		
		hypot : function() {
			for (var i = 0, j = arguments.length, k, r = 0; i < j; i++) {
				if (
					Infinity == (k = arguments[i]) ||
					Infinity == -k
				) return Infinity;
				r += k * k;
			}
			return Math.sqrt(r);
		}, // Math.hypot
		
		imul : function(a, b) {
			var
				ax = 0xFFFF & a,
				dx = 0xFFFF & b;
				return ax * dx + ((( 
					(0xFFFF & (a >> 16)) * dx +
					(0xFFFF & (b >> 16)) * ax
				) << 16) >>> 0) | 0;
		}, // Meth.imul
		
		sign : function(a) {
			return ((a > 0) - (a < 0)) || +a;
		}, // Math.sign
		
		sinh : function(a) {
			return ((a = Math.exp(a)) - 1 / a) / 2;
		},
		
		tanh : function(a, b, c) {
			return (b = Math.exp(+a)) == Infinity ? 1 : (
				(c = Math.exp(-a)) == Infinity ? -1 : (b - c) / (b + c)
			);
		},
		
		trunc : function (a) {
			return a < 0 ? Math.ceil(a) : Math.floor(a);
		}
		
	}); // polyfill Math
	
	polyfillAddStatic(Number, {
	
		EPSILON : 2.220446049250313e-16,
		MAX_SAFE_INTEGER : 9007199254740991,
		MIN_SAFE_INTEGER : -9007199254740991,
		
		isNaN : function(value) {
			return value != value;
		},
		
		'isFinite' : function(value) {
			return 'number' == typeof value && isFinite(value);
		},
		
		isInteger : function(value) {
			return Number.isFinite(value) && Math.floor(value) === value;
		},
		
		isSafeInteger : function(value) {
			return Number.isInteger(value) && Math.abs(value) <= Number.MAX_SAFE_INTEGER;
		}
		
	}); // polyfillAddStatic Number
	
	polyfillAddStatic(Object, {
	
		keysDoIntersect : function(obj1, obj2) {
			for (var i in obj1) if (i in obj2) return i;
			return false;
		},
	
		keysIntersection : function(obj1, obj2, result) {
			result = [];
			for (var i in obj1) if (i in obj2) result.push(i);
			return result;
		}
		
	}); // polyfillAddStatic Object
	
	polyfillAddPrototype(String, {
	
		SAFE_STRING_LIMIT : 0x0FFFFFF0,
		
		condense : function() {
			return this.replace(condenseRegex, ' ').trim();
		}, // String.condense
	
		endsWith : function(a, b) {
			_.Throw.isNull(this);
			if (b) _.Throw.isNaN(b);
			b = ( (!b && b !== 0) || b >= this.length ) ? b.length : b;
			return this.substr(b - a.length, a.length) == a;
		}, // String.endsWith

		htmlSpecialChars : function() {
			return this.replace(htmlSpecialCharsRegex, function(m) {
				return htmlSpecialCharsMap[m];
			});
		},
	
		jsToCssName : function() {
			_.Throw.isNull(this);
			var
				v = this.stripCamel(),
				f = this.indexOf('-');
			return ((
				(f > 0) && (f = v.substr(0,f)) &&
				(_.BROWSER_PREFIXES.indexOf(f) !== -1)
			) ? '-' : '') + this;
		}, // String.jsToCssName
		
		pad : function(a, b, c) {
			_.Throw.isNaN(a);
			if ((c = Math.abs(a) - this.length) <= 0) return this;
			b = (!b && b !== 0) ? ' ' : String(b).charAt(0);
			return a < 0 ? b.repeat(c) + this : this + b.repeat(c);
		}, // String.pad
		
		padEnd : function(a, b) {
			return a <= 0 ? this : this.pad(a, b);
		},
		
		padStart : function(a, b) {
			return a <= 0 ? this : this.pad(-a, b);
		},
		
		repeat : function(a) {
			_.Throw.isNull(this);
			_.Throw.range(a = a || 0, 0, Math.floor(String.SAFE_STRING_LIMIT / this.length));
			return Array(Math.floor(a) + 1).join(this);
		}, // String.repeat
		
		splitSelector : function(r) {
			_.Throw.isLooseFalse(this);
			r = new selectorStringChunker(this);
			r = {
				content : r.next('~'),
				special : r.next('@'),
				classes : r.next('.'),
				id : r.next('#'),
				tag : r.remains || 'div'
			};
			if (r.classes) r.classes = r.classes.replace('.', ' ');
			return r;
		}, // String.splitSelector
		
		startsWith : function(a, b) {
			_.Throw.isNull(this);
			if (b) _.Throw.isNaN(b);
			return (a = String(a)) == this.substr(b || 0, a.length);
		}, // String.startsWith

		strFrom : function(a, b) {
			return (b = b ? this.lastIndexOf(a) : this.indexOf(a)) == -1 ? false : this.substr(b + 1);
		}, // String.strFrom

		stripCamel : function(a) {
			if ('undefined' == typeof a) a = '-';
			return _.Throw.isNull(this), this.replace(stripCamelRegex, function(m) {
				return a + m.toLowerCase();
			});
		}, // String.stripCamel

		toCamel : function(a) {
			if ('undefined' == typeof a) a = '-';
			return _.Throw.isNull(this), this.replace(toCamelRegex, function(b, c, e) {
				return (e === 0 ? b : b.toUpperCase()).replace(a, '');
			});
		}, // String.toCamel

		trim : function() {
			return _.Throw.isNull(this), this.replace(trimRegex, '');
		}, // String.trim

		trimLeft : function() {
			return _.Throw.isNull(this), this.replace(trimLeftRegex, '');
		}, // String.trimLeft

		trimRight : function() {
			return _.Throw.isNull(this), this.replace(trimRightRegex, '');
		} // String.trimRight

	}); // polyfill String

	polyfillAddStatic(w, {
		'getComputedStyle' : function(element) {
			return element.currentStyle;
		}
	}); // polyfill Window

	// START JSON polyfill
	if (_.POLYFILL_JSON = !w.JSON) w.JSON = {
		parse : function(value) {
			_.Log.warn('Possibly insecure JSON polyfill call');
			return eval('(' + value + ')');
		},
		stringify : function(value, temp) {
			if (null == value) return null;
			if (!isNaN(value)) return isFinite(value) ? value.toString() : 'null';
			switch (typeof value) {
				case 'boolean': return value.toString();
				case 'object':
					if ('function' == typeof value.toJSON) return JSON.stringify(value.toJSON());
					temp = [];
					if (value instanceof Array) {
						for (var i = 0, j = value.length; i < j; i++)
							temp.push(JSON.stringify(value[i]));
						return '[' + temp.join() + ']';
					} else {
						for (var i in value)
							temp.push(JSON.stringify(i) + ':' + JSON.stringify(value[i]));
						return '{' + temp.join() + '}';
					}
			}
			return '"' + value.toString().replace(escRegex, function(c) {
				return escMap[c] || '\\u' + (c.charCodeAt(0) + 65536).toString(16).substr(1);
			}) + '"';
		}
	};
	// END JSON Polyfill
	
	/*
		START performance and requestAnimation Polyfill
		Based on code by Erik Möller. I'd link but the bloody page is gone!
		Thanks "My Opera"
	*/
	var
		perfLastTime = 0,
		perf = false == 'performance' in w ? w.performance = {} : w.performance;

	if (false == 'now' in perf) {
		var nowOffset = (
			perf.timing && perf.timing.navigationStart
		) ? perf.timing.navigationStart : new Date().getTime();
		perf.now = function() { return new Date().getTime() - nowOffset; };
	}

	if (
		!browserFix(w, 'requestAnimationFrame')
	) w.requestAnimationFrame = function(callback, now, elapsed) {
		now = perf.now();
		elapsed = Math.max(0, 50/3 - now - perfLastTime);
		perfLastTime = now + elapsed;
		return setTimeout( function() { callback(perfLastTime); }, elapsed);
	};

	if (
		!browserFix(w, 'cancelAnimationFrame') &&
		!browserFix(w, 'cancelAnimationFrame', 'CancelRequestAnimationFrame')
	) w.cancelAnimationFrame = clearTimeout;
	// END performance and requestAnimation Polyfill

	// *** END polyfills and browser object extensions ***

	/*
		Add 'elementals' class to body to say "yes, elementals is running
		handy if we need it as a CSS or other such trigger.
	*/
	_.Class.add(db, 'elementals');

})(window, document, navigator.userAgent);