/**
 * @license elementals.js (c) Jason M. Knight
 * Version 4.0.0 RC 2, 24 June 2018
 * http://www.elementalsjs.com
*/

/*
	*** MAINTAINER NOTES ***

	do NOT use "this" inside user callable objects like _.Class or _.Node as
	"this" is invalid during certain types of callbacks, such as Array.each

	Many of the polyfills are declared as variable functions so we can
	overload them. Remember, static functions cannot be overloaded.

	No, the single "=" in some statements below is NOT a typo, they're for
	testing loose value returned as a result WHILE storing that result!

	... and YES, this codebase uses "yodish" checks -- they typically result
	in a smaller gzipped filesize and pissing and moaning about them is just
	you being a pedantic know-nothing. YES, even if your stupid "linter" is
	set to bitch about them by default!
*/

var _;

(function(w, d, ua) {

	"use strict";

	if (!d.head) d.head = d.getElementsByTagName('head')[0];

	var

	// *** START local variables ***

	db = d.body,
	de = d.documentElement,
	escMap = {
		'"' : '\\"',
		'\\' : '\\\\',
		'\b' : '\\b',
		'\f' : '\\f',
		'\n' : '\\n',
		'\r' : '\\r',
		'\t' : '\\t'
	},
	/*
		Storing these regex as static reduces execution time when they are
		actually called.
	*/
	escRegex = /[\\\u0000-\u001F\u0022\u2028\u2029]/g,
	htmlSpecialCharsRegex = /[&<>"']/g,
	condenseRegex = /[\s\xA0]+/g,
	stripCamelRegex = /([A-Z])/g,
	toCamelRegex = /(\W[\w])/g,
	trimRegex = /^[\s\xA0]+|[\s\xA0]+$/g,
	trimLeftRegex = /^[\s\xA0]+/,
	trimRightRegex = /[\s\xA0]+$/,

	ua_msie = ua.match(/MSIE ([0-9\.]+)/i),
	ua_edge = ua.match(/Edge\/([0-9\.]+)/i),
	ua_rv = ua.match(/rv:([0-9\.]+)/i),

	idTracking = [],
	writeDepth = 0,
	writeView = false,

	dssContents = {
		img : 'alt',
		input : 'value',
		textarea : 'value'
	},
	dssSpecials = {
		a : 'href',
		abbr : 'title',
		acronym : 'title',
		audio : 'src',
		bdo : 'dir',
		blockquote : 'cite',
		button : 'type',
		del : 'cite',
		img : 'src',
		input : 'type',
		ins : 'cite',
		label : 'for',
		option : 'value',
		output : 'for',
		script : 'src',
		source : 'src',
		td : 'headers',
		th : 'scope',
		track : 'src',
		q : 'cite',
		video : 'src'
	},

	// *** END local variables ***

	// *** START local overwritable functions ***

	EventAdd, // defined later

	makeSelectionButton = function(
		targetElement, buttonContent, buttonParent, placement, callback, temp
	) {
		_.Throw.nonElement(targetElement);
		if (_.NON_CONTENT_TAGS.indexOf(targetElement.tagName.toUpperCase()) >= 0)
			_.Log.Error('Cannot create selection button for <', targetElement.tagName, '>');
		temp = {
			content : buttonContent,
			onclick : callback,
			'data-eNodeSelectTarget' : targetElement
		};
		temp[placement || 'before'] = buttonParent || targetElement;
		_.make('button.selectContents@button', temp);
	}, // makeSelectionButton

	// *** END overwritable functions ***

	// *** START local objects ***

	iWrite = {
		after : function(element, content) {
			element = iWriteElement(element);
			return element.parentNode.insertBefore(iWriteContent(content), element.nextSibling);
		},
		before : function(element, content) {
			element = iWriteElement(element);
			return element.parentNode.insertBefore(iWriteContent(content), element);
		},
		first : function(element, content) {
			element = iWriteElement(element);
			return element.insertBefore(iWriteContent(content), element.firstChild);
		},
		last : function(element, content) {
			element = iWriteElement(element);
			return element.appendChild(iWriteContent(content), element);
		},
		rewrite : function(element, content) {
			element = iWriteElement(element);
			_.Node.flush(element);
			return _.Node.write(element, iWriteContent(content));
		},
		replace : function(element, content) { // document as replace(oldNode, newNode);
			element = iWriteElement(element);
			return element.parentNode.replace(iWriteContent(content), element);
		}
	}, // iWrite

	/*
		polyfills is an Array of Objects, where each object contains the
		following properties:

			target : Object to apply the polyfill to

			prototype : an Object of properties to be applied as target.prototype[]

			static : an Object of properties to be applied as target[]

		Where properties are out of order, this is done so as to optimize for
		gzip compression. If that's a problem for you, learn to ^F
	*/

	polyfills = [{

		target : Array,

		prototype : {

			callbackAnd : function(a, b) {
				for (var i = 0, l = this.length; i < l; i++)
					if (!b(a, this[i])) return false;
				return true;
			}, // Array.callbackAnd

			callbackAndAll : function(a, b) {
				for (var i = 0, l = this.length, r = false; i < l; i++)
					r = b(a, this[i]) && r;
				return r;
			}, // Array.callbackAndAll

			callbackOr : function(a, b) {
				for (var i = 0, l = this.length, r = false; i < l; i++)
					r = b(a, this[i]) || r;
				return r;
			}, // Array.callbackOr

			every : function(a, b) {
				b = test_funcBVoid(a, b);
				for (var i = 0, l = this.length; i < l; i++)
					if (!a.call(b, this[i], i, this)) return false;
				return true;
			}, // Array.every

			fill : function(a, b, c) {
				tin(this);
				b = b || 0;
				if (b < 0) b += this.length;
				c = undefAssign(c, this.length);
				for (var i = b; i < c; i++) this[i] = a;
				return this;
			}, // Array.fill

			filter : function(a, b) {
				b = test_funcBVoid(a, b);
				for (var i = 0, r = [], l = this.length, e; e = this[i], i < l; i++)
					if (a.call(b, e, i, this)) r.push(e);
				return r;
			}, // Array.filter

			find : function(a, b) {
				b = test_funcBVoid(a, b);
				for (var i = 0, l = this.length, e; e = this[i], i < l; i++)
					if (a.call(b, e, i, this)) return e;
			}, // Array.find

			findIndex : function(a, b) {
				b = test_funcBVoid(a, b);
				for (var i = 0, l = this.length; i < l; i++)
					if (a.call(b, this[i], i, this)) return i;
			}, // Array.findIndex

			forEach : function(a, b) {
				b = test_funcBVoid(a, b);
				for (var i = 0, l = this.length; i < l; i++)
					a.call(b, this[i], i, this);
			}, // Array.forEach

			some : function(a, b) {
				b = test_funcBVoid(a, b);
				for (var i = 0, l = this.length; i < l; i++)
					if (a.call(b, this[i], i, this)) return true;
				return false;
			}, // Array.some

			includes : function(a, b) {
				if ((b = test_nullLength(this, b, 0)) === false) return false;
				for (var c = a != a, l = this.length, e; b < l; b++)
					if (a == (e = this[b]) || (c && e != e)) return true;
				return false;
			}, // Array.includes

			indexOf : function(a, b) {
				if ((b = test_nullLength(this, b, 0)) === false) return false;
				for (var l = this.length; b < l; b++) if (this[b] == a) return b;
				return -1;
			}, // Array.indexOf

			lastIndexOf : function(a, b) {
				if ((b = test_nullLength(this, b, this.length)) === false) return false;
				while (b > 0) if (this[--b] == a) return b;
				return -1;
			}, // Array.lastIndexOf

			map : function(a, b) {
				b = test_funcBVoid(a, b);
				for (var i = 0, r = [], l = this.length; i < l; i++)
					r.push(a.call(b, this[i], i, this));
				return r;
			}, // Array.map

			reduce : function(a, b) {
				return arrayReducer(a, b, this);
			}, // Array.reduce

			reduceRight : function(a, b) {
				return arrayReducer(a, b, this, true);
			} // Array.reduceRight

		} // Array prototypes

	}, {

		target : Date,

		static : {

			now : function() {
				return new Date().getTime();
			}

		} // Date statics

	}, {

		target : Math,

		static : {

			// 'constants'
			RAD   : Math.PI / 180,
			QUD   : Math.PI / 360,
			P2RAD : Math.PI / 50,
			DEG   : 180 / Math.PI,

			// methods

			acosh : function(a) {
				return Math.log(a + Math.sqrt(a * a - 1));
			}, // Math.acosh

			asinh : function(a) {
				return a === -Infinity ? a : Math.log(a + Math.sqrt(a * a + 1));
			}, // Math.asinh

			atanh : function(a) {
				return Math.log((1 + a) / (1 - a)) / 2;
			}, // Math.atanh

			cbrt : function(a, b) {
				b = Math.pow(Math.abs(a), 1 / 3);
				return a < 0 ? -b : b;
			}, // Math.cbrt

			clz32 : function(a) {
				return !a ? 32 : 31 - Math.floor(Math.log(a >>> 0) * Math.LOG2E);
			},

			cosh : function(a) {
				return ((a = Math.exp(a)) + 1 / a) / 2;
			},

			/*
				I am NOT polyfilling Math.expm1, type the -1 you lazy ****'s!
				Same for Math.log10, Math.logP, Math.log2
				Why the **** do those even EXIST?!?

				Skipping Math.fround, MAYBE later. Sad clown? A face fround...
			*/

			hypot : function() {
				for (var i = 0, j = arguments.length, r = 0, k; i < j; i++) {
					if (
						Infinity == (k = arguments[i]) ||
						Infinity == -k
					) return Infinity;
					r += k * k;
				}
				return Math.sqrt(r);
			}, // Math.hypot

			imul : function(a, b) {
				var
					ax = 0xFFFF & a,
					dx = 0xFFFF & b;
					return ax * dx + (((
						(0xFFFF & (a >> 16)) * dx +
						(0xFFFF & (b >> 16)) * ax
					) << 16) >>> 0) | 0;
			}, // Meth.imul

			sign : function(a) {
				return ((a > 0) - (a < 0)) || +a;
			}, // Math.sign

			sinh : function(a) {
				return ((a = Math.exp(a)) - 1 / a) / 2;
			},

			tanh : function(a, b, c) {
				return (b = Math.exp(+a)) == Infinity ? 1 : (
					(c = Math.exp(-a)) == Infinity ? -1 : (b - c) / (b + c)
				);
			},

			trunc : function (a) {
				return a < 0 ? Math.ceil(a) : Math.floor(a);
			}

		} // math Statics

	}, {

		target : Number,

		prototype : {

			floatToHex : function(toUpper) {
				_.Throw_isNaN(this);
				if (this === 0) return Object.is(this, -0) ? '-0x0p+0' : '0x0p+0';
				if (!isFinite(this)) return this < 0 ? '-' : '' + 'Infinity';
				var
					base = Math.abs(this),
					exponent = 0;
				if (base < 1) {
					do {
						exponent--;
						base *= 2;
					} while (base < 1);
				} else while (base >= 2) {
					exponent++;
					base /= 2;
				}
				base = base.toString(16);
				if (toUpper) base = base.toUpperCase();
				return (this < 0 ? '-' : '') +
					'0x' + base + 'p' + (exponent > 0 ? '+' : '') + exponent;
			}

		},

		static : {

			EPSILON : 2.220446049250313e-16,
			MAX_SAFE_INTEGER : 9007199254740991,
			MIN_SAFE_INTEGER : -9007199254740991,

			'isNaN' : function(value) {
				return value != value;
			},

			'isFinite' : function(value) {
				return 'number' == typeof value && isFinite(value);
			},

			isInteger : function(value) {
				return Number.isFinite(value) && Math.floor(value) === value;
			},

			isSafeInteger : function(value) {
				return Number.isInteger(value) && Math.abs(value) <= Number.MAX_SAFE_INTEGER;
			}

		} // Number statics

	}, {

		target : Object,

		static : {

			is : function(a, b) {
				if (a === b) return  a !== 0 || 1 / a === 1 / b;
				return a !== a && b !== b; // checks for NaN
			},

			keysDoIntersect : function(obj1, obj2) {
				for (var i in obj1) if (i in obj2) return i;
				return false;
			},

			keysIntersection : function(obj1, obj2, result) {
				result = [];
				for (var i in obj1) if (i in obj2) result.push(i);
				return result;
			}

		} // Object statics

	}, {

		target : String,

		static : {

			SAFE_STRING_LIMIT : 0x0FFFFFF0

		}, // String statics

		prototype : {

			condense : function() {
				return this.replace(condenseRegex, ' ').trim();
			}, // String.condense

			endsWith : function(a, b) {
				tin(this);
				if (b) _.Throw.isNaN(b);
				if ((!b && b !== 0) || b > this.length) b = this.length;
				return this.substr(b - a.length, a.length) == a;
			}, // String.endsWith

			htmlSpecialChars : function() {
				return this.replace(htmlSpecialCharsRegex, function(m) {
					return '&#' + m.charCodeAt() + ';';
				});
			},

			pad : function(a, b, c) {
				_.Throw.isNaN(a);
				if ((c = Math.abs(a) - this.length) <= 0) return this;
				b = (!b && b !== 0) ? ' ' : String(b).charAt(0);
				return a < 0 ? b.repeat(c) + this : this + b.repeat(c);
			}, // String.pad

			padEnd : function(a, b) {
				return a <= 0 ? this : this.pad(a, b);
			},

			padStart : function(a, b) {
				return a <= 0 ? this : this.pad(-a, b);
			},

			printf : function() {
				_.Throw.nonType(this, 'String');
				var template = String(this);
				for (var i = 0, pos, iLen = arguments.length; i < iLen; i++) {
					if (pos = template.indexOf('%') == -1) return template;
					var
						value, temp,
						before = template.substr(0, pos - 1),
						specifier = template.charAt(pos + 1),
						after = template.substr(0, pos + 1);
					switch (specifier) {
						case 'A':
						case 'a':
							value = Number(arguments[i]).floatToHex(specifier == 'A');
							break;
						case 'c':
							value = String(arguments[i].substr(0, 1));
							break;
						case 'd':
						case 'i':
							value = String(parseInt(arguments[i]));
							break;
						case 'E':
						case 'e':
							value = arguments[i].toExponential()[
								specifier == 'E' ? 'touppercase' : 'tolowercase'
							]();
						case 'F':
							/*
								decimal numbers don't HAVE upper/lower case?!?
								Why is this even a thing in C?!?
							*/
						case 'f':
							value = String(arguments[i]);
							break;
						case 'G':
						case 'g':
							value = String(arguments[i]);
							temp = arguments[i].toExponential();
							if (temp.length < value.length) value = temp;
							value = value[
								specifier == 'E' ? 'touppercase' : 'tolowercase'
							]();
							break;
						case 'H':
						case 'h':
							value = arguments[i].toString(16);
							if (toUpper) value = value.toUpperCase();
							value = '0x' + value;
							break;
						case 'o':
							value = '0' + arguments[i].toString(8);
							break;
						case 's':
							value = arguments[i];
							break;
						case '%':
							value = '%';
							break;
						case 'p': // assumes 32 bit "pointers" as per C90 spec
							value = arguments[i].toString(16).padStart(8, 0);
							break;
						/*
							NOT IMPLEMENTED
							%p -- no need for pointers in JavaScript
							%n -- writing to variables is hard when there's no pass by reference
						*/
					}
					template = start + String(value) + end;
				}
				return template;
			},

			repeat : function(a) {
				tin(this);
				_.Throw.range(a = a || 0, 0, Math.floor(String.SAFE_STRING_LIMIT / this.length));
				return Array(Math.floor(a) + 1).join(this);
			}, // String.repeat

			splitSelector : function(r) {
				r = new dssChunker(this);
				return {
					// content being last can contain any characters
					content : r.next('~'),
					// special cannot contain ~
					special : r.next('@'),
					// viewName cannot contain ~@
					viewName : r.next('*'),
					// name cannot contain ~@*
					name : r.next('^'),
					// namespace must be after class since it includes periods
					// cannot contain !@^
					ns : r.next(':'),
					// classname cannot contain ~@^: invalid characters anyways!
					className : r.next('.').replace('.', ' ').condense(),
					// id cannot contain ~@^:. -- invalid characters anyways!
					// and for the love of Christmas STOP trying to use periods in ID's
					// whaddaya freekin retahded?
					id : r.next('#'),
					// and of course tagnames are alpha only anyways so...
					tagName : r.remains.toLowerCase()
				};
			}, // String.splitSelector

			startsWith : function(a, b) {
				tin(this);
				if (b) _.Throw.isNaN(b);
				return (a = String(a)) == this.substr(b || 0, a.length);
			}, // String.startsWith

			strFrom : function(a, b) {
				b = b ? this.lastIndexOf(a) : this.indexOf(a);
				return b < 0 ? false : this.substr(b + 1);
			}, // String.strFrom

			stripCamel : function(a) {
				tin(this);
				a = undefAssign(a, '-');
				return this.replace(stripCamelRegex, function(m) {
					return a + m.toLowerCase();
				});
			}, // String.stripCamel

			toCamel : function(a) {
				tin(this);
				a = undefAssign(a, '-');
				return this.replace(toCamelRegex, function(b, c, e) {
					return (e === 0 ? b : b.toUpperCase()).replace(a, '');
				});
			}, // String.toCamel

			trim : function() {
				tin(this);
				return this.replace(trimRegex, '');
			}, // String.trim

			trimLeft : function() {
				tin(this);
				return this.replace(trimLeftRegex, '');
			}, // String.trimLeft

			trimRight : function() {
				tin(this);
				return this.replace(trimRightRegex, '');
			} // String.trimRight

		} // String prototypes

	}, {

		target : w,

		static : {

			'getComputedStyle' : function(element) {
				// ***FLAGGED FOR REVIEW***
				return element.currentStyle;
			}

		}

	}]; // polyfilla

	// *** END local objects ***

	// *** START underscore static methods and properties ***

	_ = {

		BROWSER_PREFIXES : ['ms', 'moz', 'o', 'webkit'],

		Class : {

			/*
				STILL trying to figure out how these regex are actually FASTER
				than the built-in Element.classList methods. Well, if we need to
				polyfill for old browsers, nice that the fills are faster than
				the native methods!

				I suspect it's because classList is an array-like iterable and in
				JavaScript anything that behaves like an array is SLOW AS HELL!
			*/

			add : function(element, cName) {
				if (cName instanceof Array)
					return cName.callbackOr(element, _.Class.add);
				if (_.Class.exists(element, cName)) return true;
				_.Class.set(element, (_.Class.get(element) + ' ' + cName).condense());
			}, // _.Class.add

			exists : function(element, cName) {
				return cName instanceof Array ?
					cName.callbackAnd(element, _.Class.exists) :
					newCNR(cName).test(_.Class.get(element));
			}, // _.Class.exists

			/*
				The classes could be stored in EITHER element.className or
				element.class depending on if it's an HTTP or XML element.
				IE won't let us access element.class directly -- if it doesn't
				exist legacy IE will actually halt execution!
			*/
			get : function(element) {
				return element.getAttribute('class') || element.className || '';
			}, // _.Class.get

			remove : function(element, cName) {
				if (cName instanceof Array) return cName.callbackOr(element, _.Class.remove);
				if (!_.Class.exists(element, cName)) return false;
				_.Class.set(
					element,
					_.Class.get(element).replace(newCNR(cName), '').condense()
				);
				return true;
			}, // _.Class.remove

			/*
				XML namespace elemnts such as SVG need to use setAttribute on "class"
				and in modern browsers this works for both, however this does not
				work in IE7/earlier so...
			*/
			set : function(element, cName) {
				if (_.IS_IE && _.IS_IE < 8) element.className = cName;
				else element.setAttribute('class', cName);
			}, // _.Class.set

			swap : function(element, cNameOld, cNameNew) {
				cNameOld = _.Class.remove(element, cNameOld);
				return _.Class.add(element, cNameNew) || cNameOld;
			}, // _.Class.swap

			toggle : function(element, cName) {
				if (_.Class.remove(element, cName)) return false;
				_.Class.add(element, cName);
				return true;
			} // _.Class.toggle

		}, // _.Class

		Compare : {

			string : function(a, b) {
				return _.safeString(a).localeCompare(_.safeString(b));
			},

			stringI : function(a, b) {
				return _.safeString(a, true).localeCompare(_.safeString(b, true));
			},

			numeric : function(a, b) {
				return (Number(a) || 0) - (Number(b) || 0);
			}

		}, // _.Compare

		EMPTY_TAGS : [
			'AREA', 'BASE', 'BR', 'COL', 'COMMAND', 'EMBED', 'HR', 'IMG',
			'INPUT', 'KEYGEN', 'LINK', 'META', 'PARAM', 'SOURCE', 'TRACK',
			'WBR'
		], // _.EMPTY_TAGS

		ajax : function(params) {
			switch (typeof params) {
				case 'function':
					params = { 'status' : { '200' : params } };
					// allow drop-through to object!
				case 'object':
					return (new eXMLHttp(params)).result;
			}
			tnt('Invalid Ajax declaration, parameter "handler" not a function or object!');
		}, // _.ajax

		Cookies : {

			/*
				Why the HELL has this stuff NEVER been implemented into JS
				as proper native functions?!?
			*/

			'delete' : function(name, path, domain) {
				_.Cookies.get(name) && _.Cookies.set(name, '', 0, path, domain);
			}, // _.Cookies.delete

			get : function(name) {
				return name && (name = d.cookie.match(
					new RegExp('(^|;)\\s*' + name + '\\s*=\\s*([^;]*)')
				) ? unescape(name[2]) : null);
			}, // _.Cookies.get

			set : function(name, value, time, path, domain, secure) {
				if (
					'undefined' == typeof time ||
					time === false
				) time = 34158171600000; // 6 June 3052, 10:00AM EST
				if (Number.isInteger(time)) time = new Date(time);
				if (time instanceof Date) time = time.toGMTString();
				_.Throw.nonType(time, 'string');
				d.cookie = name + '=' + escape(value) +
					'; expires=' + time +
					(path ? '; path=' + path : '') +
					(domain ? '; domain=' + domain : '') +
					(secure ? '; secure' : '');
			}, // _.Cookies.set

			setAfter : function(name, value, time, path, domain, secure) {
				_.Cookies.set(name, value, time ? Date.now() + time : false, path, domain, secure);
			} // _.Cookies.setAfter

		}, // _.Cookies

		Event : {

			add : function(e, value, callback, prevent, forceThis, parameters, temp) {
				if (callback instanceof Array) {
					paramters = callback;
					callback = parameters[0];
				} else {
					if ('function' != typeof callback) {
						temp = callback;
						if (forceThis) callback = forceThis[callback];
						else _.Log.error(
							'Attempt to assign view method "', temp, '" when no view present'
						);
						if ('function' != typeof callback) _.Log.error(
							'Unable to locate method "', temp, '" in associated view'
						);
					}
					parameters = [];
				}
				new EventAdd(
					e, value, callback,
					true === prevent ? 1 : (prevent || 0),
					forceThis || e,
					parameters
				);
			}, // _.Event.add

			PREVENT_BUBBLE : 1,
			PREVENT_PROPAGATION : 3

		}, // _.Event

		formSerialize : function(formElement) {

			_.Throw.notTagName(formElement, 'FORM');

			var result = '', temp;

			function add(name, value) {
				if (value && ('' !== value)) result +=
					(result ? '&' : '') +
					encodeURIComponent(name) +
					'=' +
					encodeURIComponent(value);
			}

			for (var i = 0, e; e = formElement.elements[i]; i++) {
				if (e.name) switch (e.tagName.toUpperCase()) {
					case 'INPUT':
						switch (e.type) {
							case 'file' :
								// to-do, add serialization for browsers that support it
								tnt('Cannot Serialize a <input type="file">');
							case 'checkbox':
							case 'radio':
								if (!e.checked) continue;
						}
					case 'BUTTON':
					case 'TEXTAREA':
						add(e.name, e.value);
					break;
					case 'SELECT':
						temp = e.options[e.selectedIndex];
						add(e.name, temp.value || _.Node.text(temp));
				}
			}

			return result;

		}, // _.formSerialize

		id : function(id) {
			return idTracking[id] || d.getElementById(id);
		},

		Load : {

			style : function(href, media) {
				if (media) {
					media = media.toLowerCase();
					if ('all' == media) _.log.warn(
						'media="all" is NOT recommended - ', href
					);
				}
				return _.make('link', {
					rel : 'stylesheet',
					href : href,
					'media' : media || 'screen,projection,tv',
					last : d.head
				});
			}, // _.Load.style

			script : function(src, attr) {
				return _.make('script@' + src, attr);
			} // _.Load.script

		}, // _.Load

		Log : {

			alert : function() {
				alert(logArgs(arguments));
			}, // _.Log.alert
			
			append : function() {
				_.make([
					[ 'br', { last : document.body } ],
					[ '~' + logArgs(arguments) ]
				]);
			}, // _.Log.append

			error : function() {
				_.Log.log.apply(null, arguments);
				throw new Error('Fatal elementals.js error, halting');
			} // _.Log.error

		}, // _.Log

		make : function(dss, commands, e, temp) {
			if (dss instanceof Array) {
				// when dss is an array, 'commands' is a view if at depth zero
				if (!writeDepth++) writeView = commands || false;
				for (var i = 0; e = dss[i]; i++) _.make.apply(_.make, e);
				if (!--writeDepth) writeView = false;
				return true;
			}
			dss = _.Throw.nonStringish(dss);
			if ('~' == dss[0]) e = d.createTextNode(dss.substr(1));
			else {
				dss = dss.splitSelector();
				if (!dss.tagName) dss.tagName = 'div';
				if (dss.ns) {
					if ('createElementNS' in d) e = d.createElementNS(dss.ns, dss.tagName);
					else {
						_.Log.warn('Legacy browsers are incapable of making namespaced elements.');
						return false;
					}
				} else {
					console.log(dss.tagName);
					e = d.createElement(dss.tagName);
				}
				/*
					using setAttribute with class means this works for both HTML
					and XML:NS!
				*/
				if (dss.className) _.Class.set(e, dss.className);
				if (dss.viewName && writeView) writeView._fields[dss.viewName] = e;
				if (dss.name) e.name = dss.name;
				if (dss.id) idTracking[e.id = dss.id] = e;
				if (dss.special && (temp = dssSpecials[dss.tagName])) {
					e[temp] = dss.special;
					if (dss.tagName == 'script') {
						if (!commands) commands = {};
						if (!Object.keysDoIntersect(iWrite, commands))
							commands.last = db;
					}
				}
				if (dss.content) {
					if (temp = dssContents[dss.tagName]) e[temp] = dss.content;
					else _.Node.write(e, dss.content);
				}
			}
			if (writeView) e['data-_view'] = writeView;
			if (commands instanceof Array) return _.Node.write(e, commands);
			else if ('object' == typeof commands) {
				var
					write = false,
					constructor = false;
				for (var name in commands) {
					var value = commands[name];
					if (name.startsWith('data-'))
						_.Node.dataSet(e, name.substr(5), value);
					else if (name.startsWith('on')) {
						if (value instanceof Array) value.unshift(e, name.substr(2));
						else value = [ e, name.substr(2), value ];
						if (!value[2]) _.Log.err(
							'No callback provided for "', name, '" event'
						);
						if (!value[3]) value[3] = false;
						if (!value[4]) value[4] = writeView;
						_.Event.add.apply(null, value);
					} else if (iWrite[name])
						write = name;
					else switch (name) {
						case 'class':
						case 'className':
							_.Class.set(e, value);
							break;
						case 'constructor':
							constructor = value;
							break;
						case 'content':
							_.Node.write(e, value);
							break;
						case 'style':
							for (var i in value) e.style[i] = value[i];
							break;
						case 'viewProperty':
							if (writeView) {
								if (value instanceof Array) writeView._templates[value[0]] = {
									parent : e,
									method : value[1]
								};
								else writeView._fields[value] = e;
							} else _.Log.log('Attempt to assign viewProperty on an element with no associated view');
							break;
						default:
							e.setAttribute(name, value);
					}
				}
				/*
					We put it on the DOM last so any attributes that can't be changed
					on the LIVE DOM in certain browsers are set first.
				*/
				if (write) _.Node.write(commands[write], e, write);
				if (constructor) constructor.call(e, {
					currentTarget : e,
					target : e,
					timeStamp : Date.now(),
					type : 'constructor'
				});
			} else if (commands && e.nodeType == 1) iWrite.last(e, commands);
			return e;
		}, // _.make
		Metrics : {

			absoluteX : function(element) {
				return element.getBoundingClientRect().left + _.Metrics.Scroll.x();
			}, // _.Metrics.absoluteX

			absoluteY : function(element) {
				return element.getBoundingClientRect().top + _.Metrics.Scroll.y();
			}, // _.Metrics.absoluteY

			height : function(element) {
				return !element || element == w ? (
					w.innerHeight || de.clientHeight || db.clientHeight
				) : element.clientHeight;
			}, // _.Metrics.height

			width : function(element) {
				return !element || element == w ? (
					w.innerWidth || de.clientWidth || db.clientWidth
				) : element.clientWidth;
			}, // _.Metrics.width

			Scroll : {

				height : function(element) {
					return (!element || element == w ? db : element).scrollHeight;
				}, // _.Metrics.Scroll.height

				width : function(element) {
					return (!element || element == w ? db : element).scrollWidth;
				}, // _.Metrics.Scroll.width

				maxX : function(element) {
					return _.Metrics.Scroll.width(element) - _.Metrics.width(element);
				}, // _.Metrics.Scroll.maxX

				maxY : function(element) {
					return _.Metrics.Scroll.height(element) - _.Metrics.height(element);
				}, // _.Metrics.Scroll.maxY

				x : function(element) {
					return !element || element == w ? (
						w.scrollX || w.pageXOffset || db.scrollLeft
					) : element.scrollLeft;
				}, // _.Metrics.Scroll.x

				y : function(element) {
					return !element || element == w ? (
						w.scrollY || w.pageYOffset || db.scrollTop
					) : element.scrollTop;
				} // _.Metrics.Scroll.y

			} // _.Metrics.Scroll

		}, // _.Metrics

		Node : {

			addCopyButton : function(
				element, buttonContent, buttonParentElement, placement
			) {
				makeSelectionButton(
					element,
					buttonContent || 'Copy',
					buttonParentElement,
					placement,
					function(e) {
						_.Select.node(e.currentTarget['data-eNodeSelectTarget']);
						_.Select.toClipboard();
					}
				);
			}, // _.Node.addCopyButton

			addSelectButton : function(
				element, buttonContent, buttonParentElement, placement
			) {
				makeSelectionButton(
					element,
					buttonContent || 'Select',
					buttonParentElement,
					placement,
					function(e) {
						_.Select.node(e.currentTarget['data-eNodeSelectTarget']);
					}
				);
			}, // _.Node.addSelectButton

			countAll : function(element, nodeMask) {
				return nCountBy(element, nodeMask, 'eachAll');
			}, // _.Node.countAll

			countChildren : function(element, nodeMask) {
				return nCountBy(element, nodeMask, 'eachChild');
			}, // _.Node.countChildren

			/*
				dataGet checks for the direct property first, if not found then
				it tries to getAttribute, returning null as false, then checking
				hasAttribute if need be to see if it exists but has no value.

				REMEMBER, window and some other elements lack getAttribute,
				so trap that!
			*/
			dataGet : function(element, name, splitResult, temp) {
				return (temp = element[name = 'data-' + name]) ? temp : (
					element.getAttribute ? (
						(temp = element.getAttribute(name)) === null ?
						false : (
							temp === '' ?
							element.hasAttribute && element.hasAttribute(name) :
							(splitResult ? temp.split(',') : temp)
						)
					) : false
				);
			}, // _.Node.dataGet

			/*
				Functions, objects, and arrays cannot be set via setAttribute,
				but we may end up passing them as data- values. (or at least it's
				convenient to do so) On the other hand many 'real' properties of
				Elements cannot be set via any means but setAttribute... so
				trapping the offending data types for direct assignemnt and
				dropping through all others to setAttribute SHOULD get the job done.

				The optional noOverwrite parameter only works when the value is
				not function, object, or array.
			*/

			dataSet : function(element, name, value, noOverwrite) {
				name = 'data-' + name;
				// functions and objects can't be done via setAttribute
				if (
					'function' == typeof value ||
					'object' == typeof value ||
					value instanceof Array
				) element[name] = value;
				// but legacy IE can be a total prick about this.
				else if (
					!noOverwrite ||
					!element.getAttribute(name)
				) element.setAttribute(name, value);
			}, // _.Node.dataSet

			/*
				YES _.Node.eachAll is recursive, but if your DOM nests so deep that
				you need to worry about this blowing out the stack, there is likely
				something  horrifying, terrifiyingly, and possibly criminally wrong
				with your bloody document!
			*/

			eachAll : function(element, callback, nodeMask) {
				if (element = element.firstChild) do {
					if (
						(_.Node.mask(element, nodeMask) && !callback(element)) ||
						(element.firstChild && !_.Node.eachAll(element, callback, nodeMask))
					) return false;
				} while (element = element.nextSibling);
				return true;
			}, // _.Node.eachAll

			eachChild : function(element, callback, nodeMask) {
				if (element = element.firstChild) do {
					if (_.Node.mask(element, nodeMask) && !callback(element)) return false;
				} while (element = element.nextSibling);
				return true;
			}, // _.Node.eachChild

			eachSome : function(element, callback, nodeMask) {
				if (element = element.firstChild) do {
					if (
						_.Node.mask(element, nodeMask) &&
						callback(element) &&
						element.firstChild
					) _.Node.eachSome(element, callback, nodeMask);
				} while (element = element.nextSibling);
			}, // _.Node.eachSome

			first : function(element, nodeMask) {
				return _.Node.walk(element, nodeMask, 'firstChild', 'nextSibling');
			}, // _.Node.first

			flush : function(element) {
				while (element.firstChild) element.removeChild(element.firstChild);
			}, // _.Node.flush

			last : function(element, nodeMask) {
				return _.Node.walk(element, nodeMask, 'lastChild', 'previousSibling');
			}, // _.Node.last

			mask : function(element, nodeMask) {
				return nodeMask === 0 ? true :
					isNaN(nodeMask = nodeMask || 1) ? (
						element.nodeType == 1 &&
						element.tagName.toUpperCase() == nodeMask.toUpperCase()
					) : ((
						element.nodeType == 0 ? 0 : 1 << (element.nodeType - 1)
					) & nodeMask) > 0;
			}, // _.Node.mask

			next : function(element, nodeMask) {
				return _.Node.walk(element, nodeMask, 'nextSibling', 'nextSibling');
			}, // _.Node.next

			prev : function(element, nodeMask) {
				return _.Node.walk(element, nodeMask, 'previousSibling', 'previousSibling');
			}, // _.Node.prev

			// reuse nodeMask in next/prev remove, saves a few bytes in minified/gz
			nextRemove : function(element, nodeMask) {
				nodeMask = _.Node.next(element, nodeMask);
				_.Node.remove(element);
				return nodeMask;
			}, // _.Node.nextRemove

			prevRemove : function(element, nodeMask) {
				nodeMask = _.Node.prev(element, nodeMask);
				_.Node.remove(element);
				return nodeMask;
			}, // _.Node.prevRemove

			// sadly legacy IE has no ChildNode.remove()
			remove : function(element) {
				element.parentNode.removeChild(element);
			}, // _.Node.remove

			/*
				Why not innerText or textContent for _.Node.text? HERE'S WHY:
				http://perfectionkills.com/the-poor-misunderstood-innerText/
				Hardcode this s*** so it's CONSISTENT! Not rocket science.
			*/

			text : function(element) {
				var result = '';
				_.Node.eachAll(element, function(node) {
					result += node.nodeValue + ' ';
					return true;
				}, _.Node.mask.TEXT);
				return result.condense();
			}, // _.Node.text

			walk : function(element, nodeMask, firstMethod, nextMethod, temp) {
				if (temp = element[firstMethod]) do {
					if (_.Node.mask(temp, nodeMask)) return temp;
				} while (temp = temp[nextMethod]);
				return false;
			}, // RE... SPECT... WALK... Whaddaya Say?!?

			write : function(element, content, method) {
				if (!method) method = 'last';
				if (content instanceof Array) {
					for (var i = 0, child; child = content[i]; i++) {
						if (child instanceof Array) {
							if (!child[1]) child[1] = {};
							else if (
								child[1] instanceof Array ||
								'object' != typeof child[1]
							) child[1] = { content : child[1] };
							if (!Object.keysDoIntersect(iWrite, child[1]))
								child[1].last = element;
							child = _.make.apply(_.make, child);
							if (method != 'last') {
								method = 'after';
								element = child;
							}
						} else iWrite[method](element, child);
					}
				} else iWrite[method](element, content);
			} // _.Node.write

		}, // _.Node

		safeString : function(stringish, toLowerCase) {
			stringish = (
				'undefined' == typeof stringish ||
				(!stringish && stringish !== 0)
			) ? '' : String(stringish).trim();
			return toLowerCase ? stringish.toLowerCase() : stringish;
		}, // _.safeString

		Select : {

			all : function() {
				_.Select.node(db);
			} // _.Select.all

		}, // _.Select

		selectTag : {

			set : function(element, value, usingText) {
				_.Throw.notTagName(element, 'select');
				var
					index = 0,
					current = _.Node.first(element, 'option');
				if (current) do {
					if (value == (usingText ? _.Node.text(current) : current.value)) {
						element.selectedIndex = index;
						return true;
					}
					index++;
				} while (current = _.Node.next(current, 'option'));
				return false;
			}, // _.selectTag.set

			get : function(element, asText) {
				_.Throw.notTagName(element, 'select');
				element = element.options[element.selectedIndex];
				return asText ? _.Node.text(element) : element.value;
			}, // _.selectTag.get

		}, // _.selectTag

		Throw : {

			isLooseFalse : function(value) {
				if (!value) tnt(value + ' should not be loose false.');
			}, // _.Throw.isLooseFalse

			isNaN : function(value) {
				tin(value);
				if (value != value) tnr('"value" must be a number');
			}, // _.Throw.isNaN

			isNull : function(value) {
				if (null === value || 'undefined' == typeof value) tnt(
					'Variable Null or Undefined'
				);
			}, // _.Throw.isNull

			nonElement : function(element) {
				tin(element);
				if (element.nodeType !== 1) tnt(
					element + ' is not a DOM Element'
				);
			}, // _.Throw.nonElement

			nonInstance : function(object1, object2) {
				tin(object1);
				if (!(object1 instanceof object2)) tnt(
					object1 + ' is not an instance of ' + object2
				);
			}, // _.Throw.nonInstance

			nonStringish : function(value) {
				if (
					!value ||
					!isNaN(value) ||
					'object' == typeof value ||
					'function' == typeof value ||
					value instanceof Array
				) tnt('"' + value + '" is not a non-empty non-numeric String');
				return String(value);
			}, // _.Throw.nonStringish

			nonType : function(value, type) {
				tin(value);
				if (type !== typeof value) tnt(
					value + ' is not a "' + type + '" type'
				);
			}, // _.Throw.nonType

			notTagName : function(element, tagName) {
				tin(element);
				tagName = tagName.toUpperCase();
				if (
					1 != element.nodeType ||
					element.tagName.toUpperCase() != tagName
				) tnt(element + ' is not a <' + tagName + '>');
			}, // _.Throw.notTagName

			range : function(value, min, max) {
				_.Throw.isNaN(value);
				if (value == Infinity) tnr('"value" must not be Inifinty');
				if (!isNaN(min) && value < min) tnr('"value" must be  >= ' + min);
				if (!isNaN(max) && value > max) tnr('"value" must be  <= ' + max);
			}, // _.Throw.range

			'type' : function(value, type) {
				if (type == typeof value) tnt(value + ' cannot be type "' + type + '"');
			} // _.Throw.type

		}, // _.Throw

		USort : {

			element : function(e, callback) {
				var sort = [], result;
				switch (e.tagName.toLowerCase()) {
					case 'tbody' :
						_.Node.eachChild(e, function(tr, result) {
							_.Node.dataSet(tr, 'USortOriginalPlace', sort.length, true);
							result = [];
							_.Node.eachChild(tr, function(a) {
								result.push(_.Node.text(a));
								return true;
							});
							sort.push({ tag : tr.cloneNode(true), data : result });
							return true;
						}, 'TR');
						break;
					case 'ul':
					case 'ol':
						_.Node.eachChild(e, function(li) {
							_.Node.dataSet(li, 'USortOriginalPlace', sort.length, true);
							sort.push({ tag : li.cloneNode(true), data : _.Node.text(li) });
							return true;
						}, 'LI');
						break;
					default: tnt(e + ' is not a TBODY, UL, or OL');
				}
				e['data-USortElementSorted'] = true;
				sort.sort(callback);
				_.Node.flush(e);
				for (var i = 0, j; j = sort[i]; i++) e.appendChild(j.tag);
			}, // _.USort.element

			restore : function(e) {
				switch (e.tagName.toLowerCase()) {
					case 'tbody':
					case 'ul':
					case 'ol':
						if (e['data-USortElementSorted']) _.USort.element(e, originalSort);
						return;
				}
				tnt(e + ' is not a TBODY, UL, or OL');
			} // _.USort.restore

		}, // _.USort

		View : function(data) {
			_.Throw.nonType(data, 'object');
			this._paintRequestId = viewPainter.register(this);
			this.dataApplyFromObject(data.properties);
			if (data.make) {
				if (data.make instanceof Array) _.make(data.make, this);
				else _.Log.error(
					'View construction error, "make" parameter must be an Array of DOM-JON or loose false'
				);
			}
			if (this._templates) {
				for (var name in this._templates) {
					var
						rows = this[name] instanceof Array ? this[name] : [ this[name] ];
						template = this._templates[name]
					this._templates[name].elements = [];
					for (var j = 0, row; row = rows[j]; j++) template.elements.push({
						index : j,
						name : name,
						element : new template.method(template.parent, j, row)
					});
				}
			}
			if (data.getValuesFromDOM) this.dataGetFromList();
			if (data.paintOnLoad) _.Event.add(w, 'load', this.paintRequest, false, this);
		} // _.View

	}; // _

	// *** END underscore static methods and properties ***

	// internal references to static underscore methods
	// reduces compressed size for some methods
	var tin = _.Throw.isNull;

	/*
		*** START local functions ***

		These functions are invisible outside of this self-instancing
		function and are meant for internal use by elementals.js ONLY.
	*/
	
	function arrayObjectsToJSON(inArray) {
		for (var i = 0, iLen = inArray.length; i < iLen; i++)
			if ('object' == typeof inArray[i])
				inArray[i] = JSON.stringify(inArray[i]);
		return inArray;
	}

	function browserFix(target, methodName, altMethodName) {
		if (target[methodName]) return false;
		if (!altMethodName) altMethodName =  methodName.charAt(0).toUpperCase() + methodName.substr(1);
		for (var i = 0, j, k; j = _.BROWSER_PREFIXES[i]; i++)
			if (target[k = j + altMethodName]) {
				target[methodName] = target[k];
				return true;
			}
		return true;
	} // browserFix

	function eXMLHttp(params, ajax) {

		if (!(ajax = this.result = w.XMLHttpRequest ? new XMLHttpRequest() : (
			newAX('Msxml2.XMLHTTP.6.0') ||
			newAX('Msxml2.XMLHTTP.3.0') ||
			newAX('Microsoft.XMLHTTP')
		))) throw new Error('Unable to create XMLHttpRequest or ActiveX XMLHTTP');

		if (params.progress) ajax.onprogress = params.progress;

		ajax.getJSON = function() {
			return JSON.parse(this.responseText);
		};

		ajax.getXMLValueById = function(id) {
			return (id = this.responseXML.getElementById(id)) ? _.Node.text(id) : null;
		};

		ajax.getXMLValuesByTagName = function(tagName, limit, matches) {
			if (!(matches = this.responseXML.getElementsByTagName(tagName))) return false;
			if (true === limit) return _.Node.text(matches[0]);
			if (!limit || matches.length < limit) limit = matches.length;
			for (var i = 0, r = []; i < limit; i++) r.push(_.Node.text(matches[i]));
			return r;
		};

		ajax.onreadystatechange  = function() {
			if (
				params.state &&
				params.state[ajax.readyState]
			) return params.state[ajax.readyState](ajax, params);
			if (4 == ajax.readyState) {
				if (params.status) {
					if (params.status[ajax.status]) return params.status[ajax.status](ajax, params);
					if (params.status.catchAll) return params.status.catchAll(ajax, params);
				}
				_.Log[params.onStatusLog || 'warn']('Unhandled AJAX Status ', ajax.status);
			} else if (params.onReadyLog) _.Log[params.onReadyLog]('Unhandled AJAX readyState ', ajax.readyState);
		};

		if (params.request) {
			ajax.open(params.request[0], params.request[1]);
			if (params.mimeType) ajax.overrideMimeType(params.mimeType);
			if (params.headers)
				for (var i in params.headers)
					ajax.setRequestHeader(i, params.headers[i]);
			ajax.send(params.request[2]);
		}

	} // eXMLHttp

	function iWriteContent(content) {
		return 'object' == typeof content ? content : d.createTextNode(content);
	}

	function iWriteElement(e) {
		if ('object' !== typeof e) e = _.id(e);
		if (e) return e;
		_.Log.error('Attempt to Write to non-element');
	}

	function logArgs(args) {
		if ((args.length > 1) && ('string' == typeof args[0])) {
			var template = args[0];
			args = arrayObjectsToJSON([].slice.call(args, 0, 1));
			return template.printf.apply(template, args);
		} else return arrayObjectsToJSON(args).join(' ');
	}

	function logTrace() {
		_.Log.log.apply(null, arguments);
		_.Log.log('This browser does not support JavaScript backtraces');
	}

	function newAX(axName) {
		try { return new activeXObject(axName); } catch (axName) { return false; }
	}

	function newCNR(cName) {
		return new RegExp('(\\s|^)' + cName + '(\\s|$)');
	}

	function nCountBy(element, nodeMask, method, i) {
		i = 0;
		_.Node[method](element, function() { return i++; }, nodeMask);
		return i;
	}

	function originalSort(a, b) {
		return _.Compare.numeric(
			_.Node.dataGet(a.tag, 'USortOriginalPlace'),
			_.Node.dataGet(b.tag, 'USortOriginalPlace')
		);
	}

	function dssChunker(str) {
		this.remains = str;
	}

	dssChunker.prototype.next = function(match, i) {
		if ((i = this.remains.indexOf(match)) >= 0) {
			match = this.remains.substr(i + match.length);
			this.remains = this.remains.substr(0, i);
			return match;
		}
		return '';
	}

	function test_funcBVoid(a, b) {
		_.Throw.nonType(a, 'function');
		return 'undefined' == typeof b ? void 0 : b;
	}

	function test_nullLength(arr, b, value) {
		tin(arr);
		return arr.length == 0 ? false : (
			'undefined' == typeof b || (b < 0 && (b += this.length) < 0)
		) ? value : b;
	}

	function tnt(message) { throw new TypeError(message); }
	function tnr(message) { throw new RangeError(message); }

	function undefAssign(e, fallback) {
		return (undefined == typeof e) ? fallback : e;
	}

	// event handler, cannot 'this' inside it!
	function viewPainter(e) {
		for (var id in viewPainter.queue) {
			var
				view = viewPainter.views[id],
				field;
			for (var name in view._fields) if (
				view.hasOwnProperty(name) &&
				(field = view._fields[name])
			) switch (field.tagName) {
				case 'input':
				case 'textarea':
					field.value = view[name];
					break;
				case 'select':
					_.selectTag.set(field, view[name]);
					break;
				default:
					iWrite.rewrite(field, view[name]);
			}
		}
		viewPainter.queue = false;
	}

	viewPainter.countId = 0;
	viewPainter.queue = false;
	viewPainter.views = {};
	viewPainter.register = function(view) {
		this.views[this.countId] = view;
		return this.countId++;
	};
	viewPainter.request = function(id, field) {
		if (!this.queue) {
			this.queue = {};
			setTimeout(viewPainter, 0);
		}
		if (!field && field !== 0) this.queue[id] = true; // all
		else {
			if (!this.queue[id]) this.queue[id] = {};
			this.queue[id][field] = true;
		}
	};

	// *** END local functions ***

	// *** START static and generated underscore properties ***

	_.IS_IE = ua_msie ? ua_msie[1] : (/Trident/i.test(ua) ? ua_rv[1] : 0);
	_.IS_EDGE = (ua_edge && !_.IS_IE) ? ua_edge[1] : 0;
	_.SVG_SUPPORTED = d.createElement('svg').getAttributeNS;

	_.Node.mask.ALL        = 0x0000;
	_.Node.mask.ELEMENT    = 0x0001;
	_.Node.mask.TEXT       = 0x0004;
	_.Node.mask.PROCESSING = 0x0040;
	_.Node.mask.COMMENT    = 0x0080;
	_.Node.mask.DOCUMENT   = 0x0100;
	_.Node.mask.DOCTYPE    = 0x0200;
	_.Node.mask.FRAGMENT   = 0x0400;

	_.NON_CONTENT_TAGS = _.EMPTY_TAGS.concat([
		'AUDIO', 'COLGROUP', 'HEAD', 'MAP', 'METER', 'PROGRESS',
		'OBJECT', 'SCRIPT', 'STYLE', 'TEXTAREA', 'VIDEO'
	]); // _.NON_CONTENT_TAGS

	// *** END static and generated underscore properties ***

	// *** START underscore prototypes ***
	
	if (IS_IE && IS_IE < 11) {
		if (w.console) {
			_.Log.log = function() {
				console.log(logArgs(arguments));
			};
			_.Log.trace = logTrace;
			_.Log.warn = function() {
				console.warn(logArgs(arguments));
			};
		}	else {
			_.Log.log = _.log.append;
			_.Log.trace = logTrace;
			_.Log.warn = function() {
				_.log.append('WARNING');
				_.log.append.apply(null, arguments);
			};
		}
	} else {
		_.Log.log = console.log;
		_.Log.trace = function() {
			console.log.apply(null, arguments);
			if (trace in console) console.trace();
			else console.log('Console.trace unsupported on this browser');
		}
		_.Log.warn = console.warn;
	}	

	_.View.prototype = {
		_fields : {},
		_templates : {},
		dataApplyFromObject : function(values) {
			_.Throw.nonType(values, 'object');
			for (var i in values) {
				if (this[i]) _.Log.error(
					'View construction error, "' + i + '" is a reserved word or already assigned.'
				);
				this[i] = values[i];
			}
		},
		dataGetFromList : function(list) {
			var list = arguments ? arguments : this._fields;
			for (var i in list) this.dataGetFrom(i, list[i]);
		},
		dataGetFrom : function(name, source) {
			switch (source.tagName) {
				case 'input':
				case 'textarea':
					return this[name] = source.value;
				case 'select':
					return this[name] = _.selectTag.get(source);
				case 'audio':
				case 'img':
				case 'video':
				case 'track':
					return this[name] = source.src;
			}
			return this[name] = _.Node.text(source);
		},
		requestPaint : function(name) {
			viewPainter.request(this._paintRequestId, name);
		}
	}

	// *** END underscore prototypes ***

	// *** START browser specific undersscore methods ***

	// START browser specific _.Event.prevent and local EventAdd object
	if (w.addEventListener) {
		EventAdd = function(element, event, callback, captureLevel, forceThis, parameters) {
			element.addEventListener(event, function(e) {
				parameters[0] = e;
				callback.apply(forceThis, parameters);
				if (captureLevel > 1) _.Event.prevent(e);
			}, captureLevel == 1);
		};
		_.Event.prevent = function(e) {
			if (e.stopPropagation) e.stopPropagation();
			if (e.preventDefault) e.preventDefault();
		};
	} else {
		_.Event.prevent = function(e) {
			e.cancelBubble = true;
			e.returnValue = false;
		};
		EventAdd = function(element, event, callback, captureLevel, forceThis, parameters) {
			element.attachEvent('on' + event, function(e) {
				e = w.event;
				e.target = w.srcElement;
				e.currentTarget = element;
				parameters[0] = e;
				callback.apply(forceThis, parameters);
				if (captureLevel > 0) {
					e.cancelBubble = true;
					if (captureLevel > 1) e.returnValue = false;
				}
			});
		};
	}
	// END browser specific _.Event.prevent and local EventAdd object

	// START browser specific _.Select.node and _.Select.none
	if (d.selection) {
		_.Select.node = function(element, temp) {
			temp = db.createTextRange();
			temp.moveToElementText(element);
			temp.select();
			return true;
		};
		_.Select.none = function() {
			d.selection.empty();
			return true;
		};
	} else if (w.getSelection) {
		_.Select.node = function(element, tempRange, tempSel) {
			tempRange = d.createRange();
			tempRange.selectNodeContents(element);
			tempSel = w.getSelection();
			tempSel.removeAllRanges();
			tempSel.addRange(tempRange);
			return true;
		};
		_.Select.none = function() {
			w.getSelection().removeAllRanges();
			return true;
		};
	} else {
		_.Select.node =
		makeSelectionButton = function() {
			_.Log.warn('selection of content not supported');
			return false;
		};
		_.Select.none = function() {
			_.Log.warn('de-selection of content not supported');
			return false;
		};
	}
	// END browser specific _.Select.node and _.Select.none

	// START browser specific _.writeToClipboard, _.Select.toClipBoard
	if (
		w.clipboardData &&
		w.clipboardData.setData
	) {
		_.writeToClipboard = function(a) {
			w.clipboardData.setData('Text', String(a));
			return true;
		};
		_.Select.toClipboard = function() {
			_.writeToClipboard(
				'function' == typeof d.getSelection ?
				String(d.getSelection()) :
				d.selection.createRange().text
			);
			return true;
		};
	} else if (d.queryCommandSupported('copy')) {
		_.writeToClipboard = function(a, b) {
			b = _.make('textarea', { content : String(a), last : db });
			b.select();
			d.execCommand('copy');
			_.Node.remove(b);
			return true;
		};
		_.Select.toClipboard = function() {
			d.execCommand('copy');
			return true;
		};
	} else {
		_.writeToClipboard = _.Select.toClipboard = function() {
			_.Log.warn('This browser does not allow JavaScript to access the clipboard');
			return false;
		};
	}
	// END browser specific _.writeToClipboard, _.Select.toClipBoard

	// START browser specific _.Query and _.Query.all
	if ('function' == typeof d.querySelectorAll) {
		_.Query = function(element, selector) {
			return element.querySelector(selector);
		};
		_.Query.all = function(element, selector) {
			// [].slice makes it return an ARRAY
			return [].slice.call(element.querySelectorAll(selector));
		};
	} else {
		var tempStyle = d.createStyleSheet();
		_.Query = function(element, selector, temp) {
			temp = _.Query.all(element, selector, 1);
			return (temp && temp.length > 0) ? temp[0] : null;
		};
		_.Query.all = function(element, selector, limit, results) {
			limit = limit || 32767;
			results = [];
			tempStyle.addRule(selector, 'fu:bar');
			function callback(e) {
				if ((limit > 0) && (e.currentStyle.fu === 'bar')) results.push(e);
				return --limit > 0;
			}
			_.Node.eachAll(element, callback);
			tempStyle.removeRule();
			return results;
		};
	}
	// END browser specific _.Query and _.Query.all

	// *** END browser specific undersscore methods ***

	// *** START polyfills and browser object extensions ***

	for (var i = 0, set; set = polyfills[i]; i++) {
		if (set.prototype)
			for (var name in set.prototype)
				if (!set.target.prototype[name])
					set.target.prototype[name] = set.prototype[name];
		if (set.static)
			for (var name in set.static)
				if (!set.target[name])
					set.target[name] = set.static[name];
	}

	polyfills = null; // free up some memory

	// START JSON polyfill
	if (_.POLYFILL_JSON = !w.JSON) w.JSON = {
		parse : function(value) {
			_.Log.warn('Possibly insecure JSON polyfill call');
			return eval('(' + value + ')');
		},
		stringify : function(value, temp) {
			if (null === value) return null;
			if (!isNaN(value)) return isFinite(value) ? value.toString() : 'null';
			switch (typeof value) {
				case 'boolean': return value.toString();
				case 'object':
					if ('function' == typeof value.toJSON) return JSON.stringify(value.toJSON());
					temp = [];
					if (value instanceof Array) {
						for (var i = 0, j = value.length; i < j; i++)
							temp.push(JSON.stringify(value[i]));
						return '[' + temp.join() + ']';
					}
					for (var i in value)
						temp.push(JSON.stringify(i) + ':' + JSON.stringify(value[i]));
					return '{' + temp.join() + '}';
			}
			return '"' + value.toString().replace(escRegex, function(c) {
				return escMap[c] || '\\u' + (c.charCodeAt(0) + 65536).toString(16).substr(1);
			}) + '"';
		}
	};
	// END JSON Polyfill

	/*
		START performance and requestAnimation Polyfill
		Based on code by Erik Möller. I'd link but the bloody page is gone!
		Thanks "My Opera"

		Updated to use webkitNow if available for legacy Safari.
	*/
	var
		perfLastTime = 0,
		perf = false == 'performance' in w ? w.performance = {} : w.performance;

	if (false == 'now' in perf) {
		if (perf.webkitNow) perf.now = perf.webkitNow; else {
			var nowOffset = (
				perf.timing && perf.timing.navigationStart
			) ? perf.timing.navigationStart : Date.now();
			perf.now = function() { return Date.now() - nowOffset; };
		}
	}

	if (
		browserFix(w, 'requestAnimationFrame')
	) w.requestAnimationFrame = function(callback, now, elapsed) {
		perfLastTime = (
			elapsed = Math.max(0, 50/3 - (now = perf.now()) - perfLastTime)
		) + now;
		return setTimeout( function() { callback(perfLastTime); }, elapsed);
	};

	if (
		browserFix(w, 'cancelAnimationFrame') &&
		browserFix(w, 'cancelAnimationFrame', 'CancelRequestAnimationFrame')
	) w.cancelAnimationFrame = clearTimeout;

	// END performance and requestAnimation Polyfill

	// *** END polyfills and browser object extensions ***

	/*
		Add 'elementals' class to body to say "yes, elementals is running
		handy if we need it as a CSS or other such trigger.
	*/
	_.Class.add(db, 'elementals');

})(window, document, navigator.userAgent);